/*
 * Glob.java
 *
 * Created on 28 juin 2004, 18:08
 */

/**
 *
 * @author  Yves
 * @version 
 */
// Classe permettant les calculs mathmatiques globaux

package glob;

import java.awt.*;
import java.awt.geom.*;
import java.util.Locale;
import java.util.StringTokenizer;
import java.text.*;

import objets.*;
import types.*;

public class Kernel {

  // Dfinition de certaines valeurs trs utilises
  public static final double ConvDegRad = Math.PI/180;
  public static final double ConvRadDeg = 180/Math.PI;
  public static final double cos30 = Math.cos(30*ConvDegRad);
  public static final double sin30 = 0.5;
  
    
  public static final double distancePointPoint(double x1,double y1,double x2,double y2)
  {
    VecteurDirecteur u = new VecteurDirecteur(x1, y1, x2, y2);
    return u.norme();
  }
  public static final double distancePointDroite(double xp,double yp,double xd,double yd,
    VecteurDirecteur u)
  {
    return Math.abs(u.y*(xd-xp)-u.x*(yd-yp))/Math.sqrt(u.x*u.x+u.y*u.y);
  }
  public static final void orthogonal(VecteurDirecteur u, VecteurDirecteur v)
  {
    v.y = u.x;
    v.x = -u.y;
  }
  public static final void intersectionDroitesSecantes(double p1_x,double p1_y,
        VecteurDirecteur u1,double p2_x,double p2_y,
        VecteurDirecteur u2, Point2D.Double res)
  {
    double d,k1,k2;

    d = u1.x*u2.y-u2.x*u1.y;
    k1 = u1.y*p1_x-u1.x*p1_y;
    k2 = u2.y*p2_x-u2.x*p2_y;
    res.x = (u1.x*k2-u2.x*k1)/d;
    res.y = (u1.y*k2-u2.y*k1)/d;
  }
  
  public static final void projetteOrtho(double x1, double y1,double p1_x,double p1_y
                     ,VecteurDirecteur u,Point2D.Double res)
  {
    VecteurDirecteur v = new VecteurDirecteur();
    orthogonal(u,v);
    intersectionDroitesSecantes(x1,y1,v,p1_x,p1_y,u,res);
  }

  public static final boolean zero(double r)
  {
    return (Math.abs(r)<1e-9);
  }
  
  // Remani pour la version 1.9
  // On teste la nullit du sinus plutot que celle du dterminant
  /* Ancienne version
  static final boolean colineaires(VecteurDirecteur u, VecteurDirecteur v)
  {
    return (zero(u.x*v.y-u.y*v.x));
  }
  */
  public static final boolean colineaires(VecteurDirecteur u, VecteurDirecteur v)
  {
    double n1 = u.norme();
    double n2 = v.norme();
    if ((n1 == 0) || (n2 == 0)) return true;
    else return (zero((u.x*v.y-u.y*v.x)/n1/n2));
  }
  // Rajout pour la version 2.0 de l'applet pour vrifier si deux vecteurs sont colinaires
  // et de mme sens. Est utilis dans coincideAvec de l'objet CDemiDroite
  public static final boolean colineairesMemeSens(VecteurDirecteur u, VecteurDirecteur v)
  {
    return (colineaires(u, v) && (u.x*v.x + u.y*v.y >= 0));
  }
  

  // Test pour savoir si des coordonnes peuvent bien tre converties en int
  public static final boolean testToile(double x, double y) {
    // A cause d'un bug de la version 1.4 qui plante pour des segments de grandes dimensions,
    // je restreins testToile  short
    //return ((x<=Integer.MAX_VALUE) && (y<=Integer.MAX_VALUE) 
    //  && (x>=Integer.MIN_VALUE) && (y>=Integer.MIN_VALUE));
    return ((x<=Short.MAX_VALUE) && (y<=Short.MAX_VALUE) 
      && (x>=Short.MIN_VALUE) && (y>=Short.MIN_VALUE));
  }

  public static final boolean appartientSegment(double x,double y,double p1x,double p1y,
                         double p2x,double p2y) {
    double k;
    double absx,absy;

    double difx = p2x-p1x;
    absx = Math.abs(difx);
    double dify = p2y-p1y;
    absy = Math.abs(dify);
    if ((absx == 0) && (absy == 0))
    {
      if ((x == p1x) && (y == p1y)) return true;
      else return false;
    }
    else
    {
      if (!(aligne(p1x,p1y,p2x,p2y,x,y)))
      return false;
      else
      {
        // Nouvelle modif
        if (absy > absx)
          k = (y-p1y)/(dify);
        else
        {
          k = (x-p1x)/(difx);
        }
        // Modification apporte pour la version 1.9.5 pour compter avec les erreurs d'arrondi
        if (((k>=0) && (k<=1)) || zero(k) || zero(1-k))
        return true;
        else return false;
      }
    }
  }

  public static final boolean aligne(double xa,double ya,double xb,double yb,
              double xc,double yc) {
    double k,nu,nv;
    VecteurDirecteur u = new VecteurDirecteur();
    VecteurDirecteur v = new VecteurDirecteur();

    u.x = xb-xa;
    u.y = yb-ya;
    v.x = xc-xa;
    v.y = yc-ya;
    nu = u.norme();
    nv = v.norme();
    // Modification apporte lors de la version 1.9.5
    //if ((nu == 0) || (nv == 0))
    if (zero(nu) || zero(nv))    
      return true;
    else
    {
      k = (u.x*v.y-v.x*u.y)/nu/nv;
      if (zero(k)) return true; else return false;
    }   
  }
  
  public static final boolean egal(CPointAncetre A, CPointAncetre B)
  {
    return ((A.x == B.x) && (A.y == B.y));
  }
  
  public static void traceQuadrilaterePlein(Graphics2D pDC, double x1, double y1, double x2, double y2,
  		double x3, double y3, double x4, double y4) {
  	int[] x = new int[5];
  	int[] y = new int[5];
  	x[0] = (int) x1;
   	y[0] = (int) y1;
  	x[1] = (int) x2;
   	y[1] = (int) y2;
  	x[2] = (int) x3;
   	y[2] = (int) y3;
  	x[3] = (int) x4;
   	y[3] = (int) y4;
  	x[4] = (int) x1;
   	y[4] = (int) y1;
  	Polygon poly = new Polygon(x, y, 5);
  	pDC.fillPolygon(poly);
  }  
  
  public static final boolean zeroAngle(double r)
  {
      return (Math.abs(r)<1e-7);
  }
  
  
  public static final boolean testAngleDroit(double angle1,double angle2)
  {
    double dif;
    dif = Math.abs(angle2 - angle1);
    return ((zeroAngle(dif - Math.PI/2)) || (zeroAngle(dif - 3*Math.PI/2)));
  }
   
  public static final void secteurSaillant(Graphics2D pDC,double xa,double ya,double xo,double yo,
            double rayon,double angle)
  {
    double xg,yg;	/// Coordonnes du sommet gauche du carr devant contenir le cercle
    int width;	/// Cot du carr contenant le cercle

    VecteurDirecteur u = new VecteurDirecteur(xo, yo, xa, ya);
    double angleDegreu = u.angleRad()/Math.PI*180;
    xg = xo - rayon;
    yg = yo - rayon;
    width = (int)Math.round(2*rayon);
    double angleArcDegre = angle/Math.PI*180;
    // Modifications par erapport au C++
    if (zero(Math.PI - angle)) angleArcDegre = 180;
    pDC.drawArc((int)Math.round(xg), (int)Math.round(yg), width, 
      width, (int)Math.round(angleDegreu), (int)Math.round(angleArcDegre));
  } 
  public static final void secteurSaillantPlein(Graphics2D pDC,double xa,double ya,double xo,double yo,
            double rayon,double angle)
  {
    double xg,yg;	/// Coordonnes du sommet gauche du carr devant contenir le cercle
    int width;	/// Cot du carr contenant le cercle

    VecteurDirecteur u = new VecteurDirecteur(xo, yo, xa, ya);
    double angleDegreu = u.angleRad()/Math.PI*180;
    xg = xo - rayon;
    yg = yo - rayon;
    width = (int)Math.round(2*rayon);
    double angleArcDegre = angle/Math.PI*180;
    // Modifications par erapport au C++
    if (zero(Math.PI - angle)) angleArcDegre = 180;
    pDC.fillArc((int)Math.round(xg), (int)Math.round(yg), width,
      width, (int)Math.round(angleDegreu), (int)Math.round(angleArcDegre));
  } 
  public static final void secteurDirect(Graphics2D pDC,double xa,double ya,double xo,double yo,
            double rayon,double angle)
  {
    if (angle < 0) angle = 2*Math.PI + angle;
    secteurSaillant(pDC, xa, ya, xo, yo, rayon, angle);
  }
  public static final void secteurDirectPlein(Graphics2D pDC,double xa,double ya,double xo,double yo,
            double rayon,double angle)
  {
    if (angle < 0) angle = 2*Math.PI + angle;
    secteurSaillantPlein(pDC, xa, ya, xo, yo, rayon, angle);
  }
  public static final void secteurRentrant(Graphics2D pDC,double xa,double ya,double xo,double yo,
            double rayon,double angle)
  {
    if ((angle > 0) || zero(angle - Math.PI)) { //secteur indirect?
      VecteurDirecteur u = new VecteurDirecteur(xo, yo, xa, ya);
      VecteurDirecteur v = new VecteurDirecteur();
      u.tourne(angle, v);
      double x1 = xo + v.x;
      double y1 = yo + v.y;
      secteurDirect(pDC, x1, y1, xo, yo, rayon, 2*Math.PI - angle);
    }
    else { // secteur direct
      secteurDirect(pDC, xa, ya, xo, yo, rayon, 2*Math.PI + angle);
    }
  }

  public static final void secteurRentrantPlein(Graphics2D pDC,double xa,double ya,double xo,double yo,
            double rayon,double angle)
  {
    if ((angle > 0) || zero(angle - Math.PI)) { //secteur indirect?
      VecteurDirecteur u = new VecteurDirecteur(xo, yo, xa, ya);
      VecteurDirecteur v = new VecteurDirecteur();
      u.tourne(angle, v);
      double x1 = xo + v.x;
      double y1 = yo + v.y;
      secteurDirectPlein(pDC, x1, y1, xo, yo, rayon, 2*Math.PI - angle);
    }
    else { // secteur direct
      secteurDirectPlein(pDC, xa, ya, xo, yo, rayon, 2*Math.PI + angle);
    }
  }
  /** Renvoie la chaine reprsentant le double donn avec decimales chiffres aprs la virgule
   * 
   * @param nombre
   * @param decimales
   * @return
   */
  public static final String chaineNombre(double nombre, int decimales) {
    // On regarde d'abord s'il faut ou pas passer en notation scientifique
    double abs = Math.abs(nombre);
    String retour;
    // if ((abs != 0) && (abs < 1e-12) || (abs > 1e12) || (decimales > 12)) {
    // Passage en notation scientifique seulement pour le nombre maxi de dcimales (12)
    // Erreur corrige pour la version 1.9.5 car la chaine pouvait devenir norme
    // et le tampon insuffisant
    // if ((decimales == 12) && (((abs != 0) && (abs < 1e-12)) || abs > 1e12)) {
    if (((decimales == 12) && (abs != 0) && (abs < 1e-12)) || (abs > 1e12)) {
      // Passage en notation scientifique.
      // Dans cette version compatible 1.4 on peut utiliser la classe java.text.NumberFormat
      StringBuffer chFormat = new StringBuffer(decimales + 4);
      chFormat.append('0');    
      chFormat.append('.');
      for (int i = 1; i <= decimales; i ++) {
        chFormat.append('#');
      }      
      chFormat.append('E');
      chFormat.append('0');
      DecimalFormat formatter = new DecimalFormat(chFormat.toString());
      formatter.setDecimalSeparatorAlwaysShown(false);
      retour = formatter.format(nombre);
    }
    else {
      StringBuffer chFormat = new StringBuffer(decimales + 2);
      chFormat.append('0');    
      chFormat.append('.');
      for (int i = 1; i <= decimales; i ++) {
        chFormat.append('#');
      }
      //Le deuxime paramtre pour viter l'emploi en France de la virgule
      DecimalFormat formatter = new DecimalFormat(chFormat.toString(), new DecimalFormatSymbols(Locale.ENGLISH));
      formatter.setDecimalSeparatorAlwaysShown(false);
      retour = formatter.format(nombre);
    }
    if (retour.compareTo("-0") == 0) return "0";
    else return retour;
  }
  
  // Calcule l'intersection de deux droites et renvoie un byte du type PositionDroites
  // donnant la nature de cette intersection.
  // Si les droites sont scantes, renvoie dans point1 les coordonnes de l'intersection
  // Remani pour la version 1.9
  // n'est appel que pour des droites existant, c'est  dire ayant des vecteurs directeurs
  // non nuls
  // Calcule l'intersection de deux droites et renvoie un byte du type PositionDroites
  // donnant la nature de cette intersection.
  // Si les droites sont scantes, renvoie dans point1 les coordonnes de l'intersection

  public static final byte intersection(double p1_x,double p1_y, VecteurDirecteur u1,double p2_x,double p2_y,
      VecteurDirecteur u2, Point2D.Double pointInt)
  {
    double d,k1,k2;
    byte position;

    d = u1.x*u2.y-u2.x*u1.y;
    // Modifi pour la version 1.9 : on teste la presque nullit du sinus et
    // non du dterminant
    if (zero(d/u1.norme()/u2.norme()))
    {
      VecteurDirecteur w = new VecteurDirecteur(p1_x,p1_y,p2_x,p2_y);
      // Il faut appeler colineaires() car w peut tre nul
      if (colineaires(u1,w)) position = PositionDroites.Confondues;
      else position = PositionDroites.Paralleles;
    }
    else
    {
      position = PositionDroites.Secantes;
      k1 = u1.y*p1_x-u1.x*p1_y;
      k2 = u2.y*p2_x-u2.x*p2_y;
      pointInt.x = (u1.x*k2-u2.x*k1)/d;
      pointInt.y = (u1.y*k2-u2.y*k1)/d;
    }
    return position;
  } 
  // Calcule l'intersection d'une droite et un cercle et renvoie un byte du type PositionDroiteCercle
  // donnant la nature de cette intersection.
  // Si ils sont scants, renvoie dans point1 et dans point2 les coordonnes des points d'intersection
  public static final byte intersectionDroiteCercle(double px,double py,VecteurDirecteur u,
      double cx, double cy, double R, Point2D.Double point1, Point2D.Double point2)
  {
    double a,b,k1,k2,k3,k4,n2,distcentredroite,dis;
    double t1,t2;// paramtre
    byte pos;

    a = u.x;
    b = u.y;
    k1 = px-cx;
    k2 = py-cy;
    k3 = a*k1+b*k2;
    k4 = a*k2-b*k1;
    n2 = a*a+b*b;
    distcentredroite = Math.abs(a*k2-b*k1)/Math.sqrt(n2);
    if (zero(distcentredroite-R))
    {
       pos = PositionDroiteCercle.Tangents;
       t1 = -k3/n2;
       point1.x = px+a*t1;
       point1.y = py+b*t1;
       point2.setLocation(point1);
       return pos;
     }
     if (distcentredroite>R)
     {
       pos = PositionDroiteCercle.Vide;
       return pos;
     }
     else
     {
       pos = PositionDroiteCercle.Secants;
       dis = n2*R*R-k4*k4;
       t1 = (Math.sqrt(dis)-k3)/n2;// formules du second degr
       t2 = (-Math.sqrt(dis)-k3)/n2;
       point1.x = px+a*t1;
       point1.y = py+b*t1;
       point2.x = px+a*t2;
       point2.y = py+b*t2;
       return pos;
     }
  }
  // Idem pour l'intersection d'une droite et d'un cercle
  public final static byte intersectionCercleCercle(double c1x,double c1y,double r1,double c2x,double c2y,double r2,
    Point2D.Double point1, Point2D.Double point2)
  {
    double d,xi,yi,xh,yh,k;
    VecteurDirecteur u = new VecteurDirecteur();
    VecteurDirecteur v = new VecteurDirecteur();

    if ((c1x == c2x) && (c1y == c2y))
    {
      if (r1 == r2)  return PositionCercleCercle.Confondus; else return PositionCercleCercle.Vide;
    }
    else
    {
      xi = (c1x + c2x)/2;
      yi = (c1y + c2y)/2; // coordonnes du milieu de M1M2;
      VecteurDirecteur u0 = new VecteurDirecteur(c1x, c1y, c2x, c2y);
      d = Kernel.distancePointPoint(c1x,c1y,c2x,c2y);
      u0.vecteurColineaire(1,u); // u colinaire  M1M2 et mme sens
      k = (r1*r1-r2*r2)/2/d;
      xh = xi + k*u.x;
      yh = yi + k*u.y;
      u.getOrthogonal(v);
      byte positionDroites = intersectionDroiteCercle(xh,yh,v,c1x,c1y,r1,point1,point2);
      switch (positionDroites)
      {
        case PositionDroiteCercle.Tangents:
          return PositionCercleCercle.Tangents;
        case PositionDroiteCercle.Secants:
          return PositionCercleCercle.Secants;
        case PositionDroiteCercle.Vide:
          return PositionCercleCercle.Vide;
        default : return PositionCercleCercle.Vide; // Pour satisfaire le compilateur
      }
    }
  }
  // calcul des indices de la plus petite valeur et de la plus grande contenues dans un tableau
  // de double. c est lengthnombre d'indices du tableau. On renvoie dans point.x l'indice
  // de la valeur minimale et dans point.y celui de la valeur maximale.
  public static final void indiceMiniMaxi(byte c,double tab[], Point point)
  {
    byte imin,imax;
    double min,max;

    min = tab[0];
    max = min;
    imin = (byte) 0;
    imax = (byte) 0;
    for (byte i = 1; i <= c; i++) 
    {
      if (tab[i]<min)
      {
        imin = i;
        min = tab[i];
      }
      if (tab[i]>max)
      {
        imax = i;
        max = tab[i];
      }
    }
    point.x = imin;
    point.y = imax;
  }
  
  // Affichage d'un caractre Grec
  //void afficheCaractereGrec(Graphics2D pDC, char[] tabcar, int indcar, int col, int lig, int indiceTailleNom) {
  //  Font fonteGrecque = FonteGrecque.getFonteGrecque(indiceTailleNom);
  //  char[] tab = new char[1];
  //  pDC.setFont(fonteGrecque);
  //  pDC.drawChars(tabcar, indcar, 1, col, lig); 
  //}
  
  
  // Renvoie la hauteur d'une ligne  afficher
  public static int hauteurLigne(Graphics2D pDC, int indiceTaille, boolean basseResolution, String ligne) {
    Font font = FonteNom.getFonteNom(indiceTaille, FonteNom.Normal, basseResolution);    
    FontMetrics fm = pDC.getFontMetrics(font);
   int dec = 0;
   int hautfont;
   if (basseResolution) {
			hautfont = fm.getAscent() + fm.getDescent() - 3;
			if (ligne.indexOf("#V(") != -1) dec = 6;
			else
			if ((ligne.indexOf("#R(") != -1) || (ligne.indexOf("#R(") != -1)) dec = 3;
    }
    else {
 	    hautfont = fm.getAscent() + fm.getDescent() - 30;
	    if (ligne.indexOf("#V(") != -1) dec = 60;
	    else
	    if ((ligne.indexOf("#R(") != -1) || (ligne.indexOf("#R(") != -1)) dec = 30;
    }

    if ((ligne.indexOf("#H(") != -1) || (ligne.indexOf("#L(") != -1))
    {
      int dec2 = FonteNom.decalageLogiquePourExposantOuIndice(indiceTaille, basseResolution);
      if (dec2 > dec) dec = dec2;
    }
    hautfont += dec;
    return hautfont;
  }

  // Renvoie la hauteur de la premire ligne  afficher dans le cas d'un commentaire
  // from d'une ou de plusieurs lignes. Ne doit pas comprendre GetDescent();
  public static int hauteurPremiereLigne(Graphics2D pDC, int indiceTaille, boolean basseResolution, String ligne) {
    Font font = FonteNom.getFonteNom(indiceTaille, FonteNom.Normal, basseResolution);    
    FontMetrics fm = pDC.getFontMetrics(font);
    int dec = 0;
    int hautfont;
    if (basseResolution) {
	    hautfont = fm.getAscent() - 3;
	    if (ligne.indexOf("#V(") != -1) dec = 6;
	    else
	    if ((ligne.indexOf("#R(") != -1) || (ligne.indexOf("#R(") != -1)) dec = 3;
    }
    else {
	    hautfont = fm.getAscent() - 30;
	    if (ligne.indexOf("#V(") != -1) dec = 60;
	    else
	    if ((ligne.indexOf("#R(") != -1) || (ligne.indexOf("#R(") != -1)) dec = 30;
   	
    }

    if ((ligne.indexOf("#H(") != -1) || (ligne.indexOf("#L(") != -1))
    {
      int dec2 = FonteNom.decalageLogiquePourExposantOuIndice(indiceTaille, basseResolution);
      if (dec2 > dec) dec = dec2;
    }
    hautfont += dec;
    return hautfont;
}
  
  // Affiche une chaine d'une seule ligne contenant ventuellemnt des caractres Grecs
  // et renvoie la colonne de fin d'affichage
  // Modifi pour la version 1.9.5 car les caractres grecs ont dj t filtrs
  public static int afficheChaineAvecCaracteresGrecs
    (Graphics2D pDC, int indiceTaille, String chaine, int col, int lig, int styleFonte,
    		boolean basseResolution) {
    Font fonte = FonteNom.getFonteNom(indiceTaille, styleFonte, basseResolution);    
    pDC.setFont(fonte);
    FontMetrics fm = pDC.getFontMetrics(fonte);
    int coldebut = col;
    pDC.drawString(chaine,col,lig);
    coldebut += fm.stringWidth(chaine);
    return coldebut;
  }
  
  // Renvoie la largeur correspondant  l'affiche de afficheChaineAvecCaracteresGrecs
  public static int largeurChaineAvecCaracteresGrecs(Graphics2D pDC, int indiceTaille, String chaine,
        int styleFonte, boolean basseResolution) {
    // Modifi pour la version 1.9.5 car les caractres grecs ont dj t filtrs
    Font fonte = FonteNom.getFonteNom(indiceTaille, styleFonte, basseResolution);    
    pDC.setFont(fonte);
    FontMetrics fm = pDC.getFontMetrics(fonte);
    return fm.stringWidth(chaine);

  }    
  // Affiche la chaine sous un radical. Renvoie la colonne d'affichage suivante.
  public static int afficheChaineSousRacine(Graphics2D pDC, int indiceTaille, String chaine,
      int col, int lig, int styleFonte, boolean basseResolution) {
    int[] sommetsFlechesx = new int[4];
    int[] sommetsFlechesy = new int[4];
    Font fonte = FonteNom.getFonteNom(indiceTaille, styleFonte, basseResolution);    
    FontMetrics fm = pDC.getFontMetrics(fonte);
    // + 3 pixels en largeur pour le radical
    // en largeur pour le radical

		// Les caractres affichs, reste  afficher le symbole radical
    int haut = fm.getAscent();
    int colaff;
    if (basseResolution) {
      colaff = afficheChaineAvecCaracteresGrecs(pDC, indiceTaille, chaine, col + 4, lig,
          styleFonte, basseResolution); // 4 pixels de plus
			sommetsFlechesx[0] = colaff;
			sommetsFlechesy[0] = lig - haut + 1;
			sommetsFlechesx[1] = col + 3;
			sommetsFlechesy[1] = lig - haut + 1;
			sommetsFlechesx[2] = col + 1;
			sommetsFlechesy[2] = lig + 1;
			sommetsFlechesx[3] = col;
			sommetsFlechesy[3] = lig - 3;
    }
    else {
      colaff = afficheChaineAvecCaracteresGrecs(pDC, indiceTaille, chaine, col + 40, lig,
          styleFonte, basseResolution); // 4 pixels de plus
			sommetsFlechesx[0] = colaff;
			sommetsFlechesy[0] = lig - haut + 10;
			sommetsFlechesx[1] = col + 30;
			sommetsFlechesy[1] = lig - haut + 10;
			sommetsFlechesx[2] = col + 10;
			sommetsFlechesy[2] = lig + 10;
			sommetsFlechesx[3] = col;
			sommetsFlechesy[3] = lig - 30;
    	
    }
		pDC.drawPolyline(sommetsFlechesx, sommetsFlechesy, 4);
    
    return colaff;
  }
  

  // Affiche la chaine sous une barre horizontal. Renvoie la colonne d'affichage suivante.
  public static int afficheChaineSousBarre(Graphics2D pDC, int indiceTaille, String chaine,
      int col, int lig, int styleFonte, boolean basseResolution) {
    Font fonte = FonteNom.getFonteNom(indiceTaille, styleFonte, basseResolution);    
    FontMetrics fmn = pDC.getFontMetrics(fonte);
    int haut;
    if (basseResolution) haut = fmn.getAscent() - 1;
    else haut = fmn.getAscent() - 10;
    int colaff = afficheChaineAvecCaracteresGrecs(pDC, indiceTaille, chaine, col, lig,
    		styleFonte, basseResolution);
    int lhaut = lig - haut;
    pDC.drawLine(colaff, lhaut, col, lhaut);
    return colaff;
  }

  // Affiche la chaine sous une flce de vecteur. . Renvoie la colonne d'affichage suivante.
  public static int afficheChaineSousFleche(Graphics2D pDC, int indiceTaille, String chaine,
      int col, int lig, int styleFonte, boolean basseResolution) {
    Font fonte = FonteNom.getFonteNom(indiceTaille, styleFonte, basseResolution);    
    FontMetrics fmn = pDC.getFontMetrics(fonte);
    int haut = fmn.getAscent();
    int colaff = afficheChaineAvecCaracteresGrecs(pDC, indiceTaille, chaine, col, lig,
    		styleFonte, basseResolution);
    int lhaut = lig - haut;
    // Si la largeur de la flche est trop petite, on la prolonge vers la gauche
    int larg = colaff - col;
    // On peut modifier col qui est passe par valeur
    int colfin = colaff;
	  if (basseResolution) {
	    if (larg < 5) {
	      col -= 5 - larg;
	      colfin ++;
	    }
	    pDC.drawLine(colfin, lhaut, col, lhaut);
	    pDC.drawLine(colfin, lhaut, colfin - 3, lhaut - 3);
	    pDC.drawLine(colfin, lhaut, colfin - 3, lhaut + 3);
	  }
	  else {
	    if (larg < 50) {
	      col -= 50 - larg;
	      colfin ++;
	    }
	    pDC.drawLine(colfin, lhaut, col, lhaut);
	    pDC.drawLine(colfin, lhaut, colfin - 30, lhaut - 30);
	    pDC.drawLine(colfin, lhaut, colfin - 30, lhaut + 30);
	  	
	  }
    return colaff;
  }

  public static char caractereSpecial(String chaine) {
    char car = 0;
    chaine.toLowerCase();
    if (chaine.compareTo("inf") == 0) car = (char)8734;
    else
    if (chaine.compareTo(">=") == 0) car = (char)8805;
    else
    if (chaine.compareTo("<=") == 0) car = (char)8804;
    //else
    //if (chaine.compareTo("+-") == 0) car = (char)177;
    else
    if (chaine.compareTo("int") == 0) car = (char)8747;
    else
    if (chaine.compareTo("<>") == 0) car = (char)8800;
    else
    if (chaine.compareTo("~") == 0) car = (char)8776;
    else
    if (chaine.compareTo("->") == 0) car = (char)8594;
    else
    if (chaine.compareTo("<-") == 0) car = (char)8592;
    else
    if (chaine.compareTo("sqrt") == 0) car = (char)8730;
    else
    if (chaine.compareTo("#") == 0) car = '#';
    else
    if (chaine.compareTo("&") == 0) car = '&';
    else
    if (chaine.compareTo("=>") == 0) car = (char)8658;
    else
    if (chaine.compareTo("mult") == 0) car = '';
    return car;
  }
    
  
  // Affichage d'une chane de plusieurs lignes ou d'une seule qui peut ventuellement contenir des
  // caractres Grecs.
  // A noter :  partir de la version 1.9.5, les caractres grecs et spciaux
  // ont t chargs et adpats comme des caractres normaux au chargement de la chane
  // Il n'y a donc plus  les traiter de faon spciale lors de l'affichage
  public static void afficheChaine(Graphics2D pDC, int indiceTaille, String chaine,
      int col, int lig, boolean basseResolution) {
    int styleFonte = FonteNom.Normal;
    // Font fonte = FonteNom.getFonteNom(indiceTaille, styleFonte);    
    // FontMetrics fm = pDC.getFontMetrics(fonte);      
    int ligaff = lig;
    StringTokenizer st1 = new StringTokenizer(chaine, "\n");
    boolean premiereLigne = true;
    while (st1.hasMoreTokens()) {
      String ligne = st1.nextToken();
  		int hauteurLigne = hauteurLigne(pDC, indiceTaille, basseResolution, ligne);
      if (premiereLigne) {
        ligaff += hauteurPremiereLigne(pDC, indiceTaille, basseResolution, ligne);
        premiereLigne = false;
      }
      else ligaff += hauteurLigne(pDC, indiceTaille, basseResolution, ligne);      
      // On dcoupe la chaine en recherchant les ventuels affichages spciaux avec
      // un signe #
      StringTokenizer st2 = new StringTokenizer(ligne, "#",true);
      boolean affspecial = false;
      int coldeb = col;
      while (st2.hasMoreTokens()) {
        String ch = st2.nextToken();
        if (ch.charAt(0) == '#') { 
          // Le prochain token commence par une affichage spcial entour par
          // des parenthses
          affspecial= true;
        }
        else {
          int longch = ch.length();
          if (affspecial) {
            affspecial = false;
            // Le caractere suivantr donne le type d'affichage
            char carsp = ch.charAt(0);
            // Cas de changement de style de police
            if (carsp == 'N' || carsp == 'G' || carsp == 'I')
            {
              switch (carsp) {
              case 'N' :
                styleFonte = FonteNom.Normal;
                break;
              case 'G' :
                styleFonte = FonteNom.Gras;
                break;
              case 'I' :
                styleFonte = FonteNom.Italique;
              }
              coldeb = afficheChaineAvecCaracteresGrecs(pDC, indiceTaille, ch.substring(1),
                coldeb, ligaff, styleFonte, basseResolution);
            }
            else {// Cas d'un affichage de racine ou de vecteur ou mesure algbrique
              if (((carsp == 'V') || (carsp == 'A') || (carsp == 'R')
                                 || (carsp == 'H') || (carsp == 'L')) && (longch > 1)) {
                // On commence par rechercher la parenthse ouvrante. S'il n'y en a pas,
                // affichage normal
                if (ch.charAt(1) != '(') {
                  Kernel.afficheChaineAvecCaracteresGrecs(pDC, indiceTaille, ch, coldeb,
                    ligaff, styleFonte, basseResolution);
                  coldeb += Kernel.afficheChaineAvecCaracteresGrecs(pDC, indiceTaille, ch, coldeb,
                    ligaff, styleFonte, basseResolution);
                }
                else {
                  // On recherche la parenthse fermante correspondante
                  int indfin;
                  int indparf = ch.indexOf(')');
                  if (indparf == -1) {
                    indfin = longch - 1;
                    indparf = longch;
                  }
                  else indfin = indparf - 1;
                  switch (carsp) {
                    case 'V' :
                      coldeb = afficheChaineSousFleche(pDC, indiceTaille, ch.substring(2, indfin + 1),
                        coldeb, ligaff, styleFonte, basseResolution);
                      break;
                    case 'A' : 
                      coldeb = afficheChaineSousBarre(pDC, indiceTaille, ch.substring(2, indfin + 1),
                        coldeb, ligaff, styleFonte, basseResolution);
                      break;
                    case 'R' :
                      coldeb = afficheChaineSousRacine(pDC, indiceTaille, ch.substring(2, indfin + 1),
                        coldeb, ligaff, styleFonte, basseResolution);
                      break;
                    case 'H' :
                      {
                        String chexp =ch.substring(2, indfin + 1); 
                        int indiceTailleExposant = FonteNom.indiceTaillePourExposantOuIndice(indiceTaille);
                        int hLigneExposant = hauteurLigne(pDC, indiceTailleExposant, basseResolution, chexp);
                        // Modification pour version 1.9.5
                        int ligexp = ligaff - hauteurLigne + 3 -
                          FonteNom.decalageLogiquePourExposantOuIndice(indiceTaille, basseResolution) + hLigneExposant; 
                        coldeb = afficheChaineAvecCaracteresGrecs
                          (pDC, indiceTailleExposant, chexp, coldeb + 1, ligexp, styleFonte, basseResolution);
                        break;
                      }
                    case 'L' :
                      {
                        String chexp =ch.substring(2, indfin + 1); 
                        int indiceTailleIndice = 
                          FonteNom.indiceTaillePourExposantOuIndice(indiceTaille);
                        //int hLigneExposant = HauteurLigne(pDC, indiceTailleExposant, chexp);
                        int ligexp = ligaff +
                          FonteNom.decalageLogiquePourExposantOuIndice(indiceTaille, basseResolution); 
                        coldeb = afficheChaineAvecCaracteresGrecs
                          (pDC, indiceTailleIndice, chexp, coldeb + 1, ligexp, styleFonte, basseResolution);
                      }                    
                  } ///switch
                  // On affiche normalement ce qui suit s'il y a quelquechose
                  if (indparf < longch - 1) {
                    coldeb = afficheChaineAvecCaracteresGrecs(pDC, indiceTaille,
                      ch.substring(indparf + 1, longch), coldeb, ligaff, styleFonte, basseResolution);
                  }                
                }
              }
            }
          }
          else { // affichage normal
            coldeb = afficheChaineAvecCaracteresGrecs(pDC, indiceTaille, ch, coldeb, ligaff, styleFonte, basseResolution);
          }
        }
      } // while (st2...
    } // while (st1
  }
  
  // Mthode renvoyant dans rectangle (width et height) les dimensions du rectangle contenant une chaine affiche
  //  l'cran en tenant compte des caractres grecs et spciaux.
  public static void dimensionsChaine(Graphics2D pDC, int indiceTaille, String chaine,
    Rectangle rectres, boolean basseResolution) {
    int styleFonte = FonteNom.Normal;
    int largeur = 0;
    int hauteur = 0;
    StringTokenizer st1 = new StringTokenizer(chaine, "\n");
    while (st1.hasMoreTokens()) {
      String ligne = st1.nextToken();
      hauteur += hauteurLigne(pDC, indiceTaille, basseResolution, ligne);
      // A revoir plus tard pour les caractres spciaux;
      // On dcoupe la chaine en recherchant les ventuels affichages spciaux avec
      // un signe #
      StringTokenizer st2 = new StringTokenizer(ligne, "#",true);
      boolean affspecial = false;
      int coldeb = 0;
      while (st2.hasMoreTokens()) {
        String ch = st2.nextToken();
        if (ch.charAt(0) == '#') { 
          // Le prochain token commence par une affichage spcial entour par
          // des parenthses
          affspecial= true;
        }
        else {
          int longch = ch.length();
          if (affspecial) {
            affspecial = false;
            // Le caractere suivantr donne lengthtyme d'affichage
            char carsp = ch.charAt(0);
            if (carsp == 'N' || carsp == 'G' || carsp == 'I')
            {
              switch (carsp)
              {
              case 'N' :
                styleFonte = FonteNom.Normal;
                break;
              case 'G' :
                styleFonte = FonteNom.Gras;
                break;
              case 'I' :
                styleFonte = FonteNom.Italique;
              }
              coldeb += largeurChaineAvecCaracteresGrecs
                (pDC, indiceTaille, ch.substring(1), styleFonte, basseResolution);
            }
            else { 
              // Cas d'un affichage de racine ou de vecteur ou mesure algbrique
              if (((carsp == 'V') || (carsp == 'A') || (carsp == 'R')
                                 || (carsp == 'H') || (carsp == 'L')) && (longch > 1)) {            
                // On commence par rechercher la parenthse ouvrante. S'il n'y en a pas,
                // affichage normal
                if (ch.charAt(1) != '(') {
                  coldeb += largeurChaineAvecCaracteresGrecs(pDC, indiceTaille, ch, styleFonte, basseResolution);
                }
                else {
                  // On recherche la parenthse fermante correspondante
                  int indfin;
                  int indparf = ch.indexOf(')');
                  if (indparf == -1) {
                    indfin = longch - 1;
                    indparf = longch;
                  }
                  else indfin = indparf - 1;
                  switch (carsp) {
                    case 'V' :
                    case 'A' :
                      coldeb += largeurChaineAvecCaracteresGrecs(pDC, indiceTaille,
                        ch.substring(2, indfin + 1), styleFonte, basseResolution);
                      break;
                    case 'R' : // 4 pixels de plus en largeur pour les caratres grecs
                      coldeb += 4 + largeurChaineAvecCaracteresGrecs(pDC, indiceTaille,
                        ch.substring(2, indfin + 1), styleFonte, basseResolution) + 4;
                      break;
                  case 'H' :
                  case 'L' :
                    {
                      String chexp =ch.substring(2, indfin + 1); 
                      int indiceTailleExposant = FonteNom.indiceTaillePourExposantOuIndice(indiceTaille);
                      coldeb += 1 + largeurChaineAvecCaracteresGrecs
                        (pDC, indiceTailleExposant, chexp, styleFonte, basseResolution);
                      break;
                    }
                  } ///switch
                  // On affiche normalement ce qui suit s'il y a quelquechose
                  if (indparf < longch - 1) {
                    coldeb += largeurChaineAvecCaracteresGrecs(pDC, indiceTaille,
                        ch.substring(indparf + 1, longch), styleFonte, basseResolution);
                  }                
                }
              }
            }
          }
          else { // affichage normal
            coldeb += largeurChaineAvecCaracteresGrecs(pDC, indiceTaille, ch, styleFonte, basseResolution);
          }
        }
      } // while (st2...
      if (coldeb > largeur) largeur = coldeb;
    } // while (st1
    rectres.width = largeur;
    rectres.height = hauteur + 1;
  }  
  // Mthode enlevant d'une chaine les retours charriot et renvoyant le rsultat
  public static final String sansCR(String chaine) {
    String res = new String();
    StringTokenizer st1 = new StringTokenizer(chaine, "\r");
    while (st1.hasMoreTokens()) {
      String str = st1.nextToken();
      res = res.concat(str);
    }
    return res;
  }
  
  // Pour la compatibilit avec la version C++ (problmes dcalage en hauteur pour les
  // commentaires ou macros conteenant une flche de vecteur ou une racine carre
  // Cette mthode recherche si la premire ligne d'une chaine contient ou pas des
  // caractre surmonts d'une flche, une barre de mesure algbrique ou une racine carr
  // et renvoie le dcalage en hauteur pour assurer la compatibilit avec l'affichage
  // ralis en C++
  public static int decalagePourCompatibilite(String chaine) {
    if (chaine.length() == 0) return 0;
    StringTokenizer st1 = new StringTokenizer(chaine, "\n");
    String ligne = st1.nextToken();
    if (ligne.indexOf("#V(") != -1) return -5;
    else {
      if (ligne.indexOf("#R(") != -1) return -3;
      else {
        if (ligne.indexOf("#A(") != -1) return -1;
        else return 0;
      }
    }
  }
  //  Cr pour version 1.8
  public static double mesurePrincipale(double angrad)
  {
    if (zeroAngle(Math.abs(angrad) - Math.PI)) return Math.PI;
    return angrad - 2*Math.PI*Math.floor((angrad + Math.PI)/(2*Math.PI));
  }
  
  public static int codeCarGrec(char car) {
    switch (car) {
      case 'a' :
        return 0x3b1;
      case 'b' :
        return 0x3b2;
      case 'c' :
        return 0x3c7;
      case 'd' :
        return 0x3b4;
      case 'e' :
        return 0x3b5;
      case 'f' :
        return 0x3d5;
      case 'g' :
        return 0x3b3;
      case 'h' :
        return 0x3b7;
      case 'i' :
        return 0x3b9;
      case 'j' :
        return 0x3c6;
      case 'k' :
        return 0x3ba;
      case 'l' :
        return 0x3bb;
      case 'm' :
        return 0x3bc;
      case 'n' :
        return 0x3bd;
      case 'o' :
        return 0x3bf;
      case 'p' :
        return 0x3c0;
      case 'q' :
        return 0x3b8;
      case 'r' :
        return 0x3c1;
      case 's' :
        return 0x3c3;
      case 't' :
        return 0x3c4;
      case 'u' :
        return 0x3c5;
      case 'v' :
        return 0x3d6;
      case 'w' :
        return 0x3c9;
      case 'x' :
        return 0x3be;
      case 'y' :
        return 0x3c8;
      case 'z' :
        return 0x3b6;
      case 'D' :
        return 0x394;
      case 'F' :
        return 0x3a6;
      case 'G' :
        return 0x393;
      case 'S' :
        return 0x3a3;
      case 'W' :
        return 0x3a9;
      default :
        return 0;
    }
  }
  
  // Rajout pour la version 1.9.5
  // Renvoie une chaine correspondant la la chane initiale dans laquelle les
  // caractres grecs sous la forme &caractre sont remplacs par le caractre
  // unicode correspondant
  public final static String chaineAvecCaracteresGrecsUnicode(String ch) {
    StringBuffer chret = new StringBuffer(ch.length());
    boolean carGrec = false;
    StringTokenizer st1 = new StringTokenizer(ch, "&",true);
    while (st1.hasMoreTokens()) {
      String c = st1.nextToken();
      if (c.charAt(0) == '&') {
        carGrec = true;
      }
      else {
        if (carGrec) {
          carGrec = false;
          int lon = c.length();
          char car = (char)codeCarGrec(c.charAt(0));
          chret.append(String.valueOf(car));
          if (lon > 1) chret.append(c.substring(1));
        }
        else {
          chret = chret.append(c);
        }
      }
    }
    return chret.toString();
  }
  
  // Rajout pour la version 1.9.5
  // Renvoie une chaine correspondant la la chane initiale dans laquelle les
  // caractres spciaux sous la forme #C(chaine de caractres) sont remplacs par le caractre
  // unicode correspondant
  public final static String chaineAvecCaracteresSpeciauxUnicode(String chaine) {
    StringBuffer chret = new StringBuffer(chaine.length());
    StringTokenizer st = new StringTokenizer(chaine, "#",true);
    boolean affspecial = false;
    while (st.hasMoreTokens()) {
      String ch = st.nextToken();
      if (ch.compareTo("#") == 0) { 
        // Le prochain token commence par une affichage spcial entour par
        // des parenthses
        affspecial= true;
      }
      else {
        if (affspecial) {
          affspecial = false;
          int longch = ch.length();
          if (longch <= 2) chret.append('#' + ch);
          else {
            if (ch.substring(0, 2).compareTo("C(") == 0) {
              // On cherche la parenthse correspondante
              int indfin;
              int indparf = ch.indexOf(')');
              if (indparf == -1) indfin = longch - 1;
              else indfin = indparf - 1;
              char car = caractereSpecial(ch.substring(2, indfin + 1));
              if (car != 0) chret.append(String.valueOf(car));
              if (indparf <= longch - 1) chret.append(ch.substring(indparf+1));
              else chret.append('#' + ch);
            }
            else chret.append('#' + ch);
          }
        }
        else chret.append(ch);
      }
    }
    return chret.toString();
  }
  
}
  