/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import types.*;

/**
 *
 * @author Yves
 */
/**
  Nouveaut pour la version 1.9.5
  Pour pouvoir implmenter le calcul d'une drive (ce qui implique
  la drive d'une compose) il faut crer un nouveau type d'objet
  descendant de CCalculBase qui maintient un objet de type CCalculBase
  L'un correspond  la formule de la fonction et l'autre  la formule de l'oprande
  C'est le destructeur de ce nouveau type d'objet qui se chargera
  de dtruire les deux objets de type CCAclculBase encapsuls
  Il est aussi ncessaire de pouvoir cloner un objet de type CCalculBase
  pour pouvoir recopier des noeuds entiers dans la calcul d'une drive.
  Ce type d'objet n'est pas srializ dans le flux car le calcul de drive
  est cr  chaque chargement de la figure
 */
public class CAppelFonctionInterne extends CCalculBase {

	private CCalculBase fonctionInterne;
	private CCalculBase operande;

  /** Creates a new instance of CAppelFonctionInterne */
  public CAppelFonctionInterne(CCalculBase fonctionInterneInit, CCalculBase operandeInit) {
    fonctionInterne = fonctionInterneInit;
    operande = operandeInit;
  }
  
  @Override
  protected long nature() {
  	return natAppelFonctionInterne;
  }
    
 /// resultat sera redfinie pour tous les descendants pouvant ne pas exister
  public double resultat(boolean infoRandom) throws ErreurCalculException {
    return fonctionInterne.resultatFonction(infoRandom,operande.resultat(infoRandom));
  }
  
  @Override
  public double resultatFonction(boolean infoRandom, double valeurparametre) throws ErreurCalculException {
    return fonctionInterne.resultatFonction(infoRandom,
      operande.resultatFonction(infoRandom, valeurparametre));
  }
  
  // Ajout version 3.0
  @Override
  public double resultatFonction(boolean infoRandom, double[] valeurparametre) throws ErreurCalculException {
    return fonctionInterne.resultatFonction(infoRandom,
      operande.resultatFonction(infoRandom, valeurparametre));
  }
  
  public CCalculBase getCopie() { 
    return new CAppelFonctionInterne(fonctionInterne.getCopie(), operande.getCopie());
  }
  
  public boolean dependDeVariable(int ind)
  {
    return operande.dependDeVariable(ind);
  }
  
  // Il faut dfinir getClone car elle est abstract dans CCalcul
  // mais elle est inutile car ce type d'objet n'est pas enregistr dans un flux
  public CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    return null;
  }
  
  @Override
	public boolean estConstant() {
		return fonctionInterne.estConstant() && operande.estConstant();
	}
  
  @Override
  public boolean deriveePossible(int indiceVariable) {
  	return fonctionInterne.deriveePossible(indiceVariable) && operande.deriveePossible(indiceVariable);
  }
  
  // Ajout version 4.6
  public CCalculBase calculAvecValeursRemplacees() {
  	return new CAppelFonctionInterne(fonctionInterne, operande.calculAvecValeursRemplacees());
  }
  
  @Override
	public CCalculBase membreGauche() {
  	return fonctionInterne.membreGauche();
	}
	
	// Ajout version 4.6
	public CCalculBase membreDroit() {
  	return fonctionInterne.membreDroit();
	}
  
  
}
