/*
 * CElementBase.java
 *
 * Created on 16 juin 2004, 09:34
 */

/**
 *
 * @author  Yves
 * @version
 */
package objets;

import java.io.*;
import java.util.ListIterator;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public abstract class CElementBase extends CObjetBase {
  public boolean existe;
  // Membres rajouts pour la version 2.5
  public CImplementationProto impProto; // null si n'est pas un objet d'une implmentation de prototype
  // Trs important : lors du chargement d'un prototype, comme on ne peut pas faire pointer par exemple
  // un pointeur vers un CRepere vers un lement gnrique on est oblig de crer un nouveau repre
  // Mais lors de l'implmentatiion du prototype, dans indexOf ne nouvel objet doit donner indexOf
  // de l'lment gnrique.
  // Voir CListeObjets.get et CPrototype.indexOf pour comprendre
  public CElementGenerique elementGeneriqueAssocie;
  public boolean estElementFinal; // true si est lement final d'une implmentation de prototype
  /** Creates new CElementBase */
  public CElementBase() {
  	elementGeneriqueAssocie = null;
  }
  public CElementBase(CListeObjets liste, CImplementationProto impProtoInit,boolean estElementFinalInit) {
    super(liste);
    elementGeneriqueAssocie = null;
    impProto = impProtoInit;
    estElementFinal = estElementFinalInit;
  }
  
  /**
   * Contrairement  la version C++, la nature des objets des lments non graphiques sera NAucunObjet
   * Contrairment  la version C++ (la fonction n'existait pas en C++) renvoie uniquement la nature graphique de l'objet
   */
  public long getNature() {
    return NatureObjet.NAucunObjet;
  }
  
  /**
   * Fonction renvoyant la nature d'objet autorise pour dsigner lors de la construction
   * un objet quivalent  l'objet mais construit autrement
   * Spcifique applet
   * @return
   */
  public long naturePourConst() {
    long nat = getNature();
    if ((nat & NatureObjet.NToutPoint) != 0) return NatureObjet.NToutPoint;
    else return nat;
  }
  
  /**
   * Ajout pour la version java. Pour une valeur numrique renverra la chane la reprsentant et pour un repre
   * la chane le reprsentant (par exemple "(O,I,J)")
   * @return
   */
  public String nom() {
  	return "";
  }
  /**
   * Fonction n'existant pas dans la version C++.
   * Renvoie la nature de calcul de l'objet.
   * Si l'objet n'est pas de type calcul renvoie NAucunCalcul qui est nul.
   * A redfinir pour tous les objets de type calcul
   * @return
   */
  public long getNatureCalcul() {
  	return NatureCalcul.NAucunCalcul;
  }

  /**
   * Fonction n'existant pas en C++ et charge de mettre  jour l'objet quand un lment dont il dpend a t modifi.
   * Devra tre appel par les lieux de points et lieux d'objets pour remettre  jour les liste qu'ils utilisent
   * de faon interne
   */
  public void metAJour() {
  }
  
  /**
   * Fonction renvoyant la chane  afficher dans la bote de dialogue d'historique
   * pour dcrire l'objet.
   * A redfinir pour les descendants
   * @return
   */
  public String infoPourHistorique() {
    return "";
  }
    
  /**
   * Fonction ajoutant  list les lments qui ont gnr l'objet.
   * A redfinir pour les descendants.
   * Par exemple pour une droite passant par deux points, ajoutera  la liste
   * un pointeur sur ces deux points.
   * @param frame ce paramtre sert quabd on rahoute des lmnts provisoires comme pour un barycentre
   * @param liste
   */
  public void ajouteAntecedents(CListeObjets liste) {
    return;
  }
  
  /**
   * La mthode suivante n'existait pas dans la version C++ et servira  crer un clone
   * de tout objet dans une autre liste
   * Elle n'est plus abstract  partir de la version 1.9.5
   * pour viter de la redfinir dans les lmnts anctres des lments pouvant crer une trace d'objets
   */
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible){
    return null;
  }
  
  /**
   * Procdure servant  cloner l'tat d'un objet identique au lieu de le positionner
   * Seuls les membres pouvant tre modifis par le dplacement d'un point
   * ou l'excution d'une macro doivent tre clons.
   */
  public void setClone(CElementBase ptel) {
    existe = ptel.existe;
  }
  
  /**
   * Recalcul de l'objet
   * @param infoRandom : true si les tirages alatoires sont permis
   * @param dimfen : dimension de la fentre contenant la figure
   */
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = true;
  }
  
  public boolean dependDe(CElementBase p) {
  	boolean res = false;
  	// Modifi pour la version 2.5 pour qu'un objet final ou intermdiaire
  	// d'une implmentation de prototype dpende de cette implmentation
  	if (impProto != null) {
  		if (impProto == p) res = true;
  	}
  	res = res || (p == this);
  	return res;
  }
  
  public boolean dependDeAuMoinsUn(CListeObjets liste) {
  	boolean res = false;
  	ListIterator<CElementBase> li = liste.listIterator();
  	while(li.hasNext()) {
  		CElementBase el = li.next();
  		res = res || dependDe(el);
  	}
  	return res;
  }
    
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this);
  }
  public boolean dependDePourCapture(CElementBase p) {
    return dependDe(p);
  }
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
   **/
  public boolean confonduAvec(CElementBase p) {
    return false;
  }
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est graphiquement (presque) confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne de faon  tester la validit de la construction.
   **/
  public boolean coincideAvec(CElementBase p) {
    return false;
  }
    
  /** Fonction replaant un pointeur sur un point par un autre point
   *  Utilise entre autres dans le reclassement d'objets.
   * A redfinir pour chacun des descendants maintenant un pointeur sur un point.
   **/
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
  }
  
  /**
   * fonction n'existant pas en C++ et renvoyant un pointeur vers l'antcdent direct de l'objet.
   * Si l'objet est un objet qui ne ncessite pas lors de sa cration la cration
   * d'objets supplmentaires, renvoie this et sinon envoi un pointeur sur le premier
   * de ces objets supplmentaires crs.
   * Par exemple, pour un point li  un bipoint (intersection de deux cercles
   * ou d'un cercle et une droite) renverra un pointeur sur le bipoint
   * cr avant ce CPointLieBipoint. Pour une objet image, renverra un pointeur
   * vers la transformation utilise etc ...
   */
  public CElementBase antecedentDirect() {
    return this;
  }
  /*
   * Fonction rajoute pour la version 2.5
   **/
   // Ajout pour la version 2.5
  public boolean estElementIntermediaire() {
    return !estElementFinal && (impProto != null);
  }
  
  /**
   * Fonction qui renverra true seulement pour les mesures utilisant point
   * car alors ce point ne doit pas tre renomm avec un nom vide
   * @return
   */
  public boolean nomIndispensable(CElementGraphique el) {
    return false;
  }
  
  
  /** Fonction devant tre redfinie pour tous les objets images par une transformation
   *  Sera utilise lorsqu'on modifie une image par une transformation pour que cette image
   *  soit associe  une autre transformation lrsque d'autres objets taient des images par
   *  cette mme transformation
   */
  public void associeATransformation(CTransformation trans) {
  }
	
	/**
	 * Adaptation de certains objets pour passer en haute rsolution avant
	 * impression notamment
	 */
	public void adaptePourHauteResolution() {
	}
	
	/**
	 * Fonction renvoyant true si l'objet dpend des positions gnres pas les dplacements d'un point li
	 * A redfinir pour les lieux de points et lieux d'objets et macros utilisant les dplacements d'un point li
	 * @param pointlie
	 * @return
	 */
	public boolean estGenereParPointLie(CPointLie pointlie) {
		return false;
	}
	  
  public static long natureGraphiquePourImplementationProto(long nat) {
  	if ((nat & (NatureObjet.NToutPoint - NatureObjet.NPointLie)) != 0) return NatureObjet.NToutPoint;
  	else return nat;
  }
  
  public static long natureObjetNonGraphiquePourImplementationProto(long nat) {
  	if (nat == NatureCalcul.NCalculReel) return NatureCalcul.NTouteValeurReelle;
  	if (nat == NatureCalcul.NCalculComplexe) return NatureCalcul.NTouteValeurComplexe;
  	// Dans les autres cas mme nature attendue
  	return nat;
  }
  
  /**
   * Fonction rajoute  la version java et renvoyant true si l'objet ncessite une longueur unit pour fonctionner
   * A redfinir pour ce type d'objet : mesure de longueur, d'aire, cercle par rayon
   * @return
   */
  public boolean utiliseLongueurUnite() {
  	return false;
  }
  
 public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    if (listeProprietaire.numeroVersion <= 3) {
      estElementFinal = false;
      impProto = null;
    }
    else {
      estElementFinal = inps.readBoolean();
      int ind1 = inps.readCInt();
      impProto = (CImplementationProto) (list.get(ind1));
    }
  }
  // Nouvelles procdures de serialization java
  /* Chargement depuis un flux java */
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    estElementFinal = inps.readBoolean();
    int ind1 = inps.readInt();
    impProto = (CImplementationProto) (list.get(ind1));
  }
  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeBoolean(estElementFinal);
    int ind1;
    if (impProto == null) ind1 = -1;
    else ind1 = list.indexOf(impProto);
    oups.writeInt(ind1);
  }
}
