/*
 * CHomothetie.java
 *
 * Created on 9 juillet 2004, 18:36
 */

/**
 *
 * @author  Yves
 * @version 
 */
package objets;

import java.awt.geom.*;
import java.io.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CHomothetie extends CTransformation {
	CPointAncetre centre;
	public CValeurAssocieeAVariable rapport;
  // Java : modification par rapport au C++ : on stocke la valeur numrique du rapport
  // pour ne pas avoir  la recalculer plus tard
  double valeurRapport;

  /** Creates new CHomothetie */
  public CHomothetie() {
  }
  
  @Override
	public int natureTransformation() {
		return homothetie;
	}
  
  // Conctructeur avec un rapport constant non ditable pour les objets visuels
  public CHomothetie(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    CPointAncetre centreInit, double rap) {
    super(liste, impProtoInit, estElementFinalInit);
    rapport = new CValeurAssocieeAVariable(liste, rap);
    centre = centreInit;
  }
  
  // Constructeur pour clonage
  public CHomothetie(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    CPointAncetre centreInit, CValeurAssocieeAVariable rapportInit) {
    super(liste, impProtoInit, estElementFinalInit);
    rapport = rapportInit;
    centre = centreInit;
  }

  String designationTransf() {
    return textes.Chaines.chinfo22;
  }  

  @Override
  void AjouteElementsVisuelsPourHist(CListeObjets liste) {
    liste.add(centre);
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(centre);
    int ind2 = listeSource.indexOf(impProto);        
    CValeurAssocieeAVariable rapportClone = (CValeurAssocieeAVariable) rapport.getClone(listeSource, listeCible);
    CElementBase ptelb = new CHomothetie(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      rapportClone);
    return ptelb;
  }

  String complementPourHist() {
    return "\n" + textes.Chaines.chinfo28 + " " + rapport.chaineInfo();
  }
  
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || centre.dependDe(p) || rapport.dependDe(p);
  }

  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || centre.dependDePourBoucle(p) || rapport.dependDePourBoucle(p));
  }

  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	rapport.positionne(infoRandom, dimfen);
    existe = centre.existe && rapport.existe();
    if (existe) valeurRapport = rapport.rendValeur(infoRandom);
  }

  public boolean confonduAvec(CElementBase p) {
    CHomothetie pt;
    if (p.getClass() == this.getClass()) {
      pt = (CHomothetie) p;
      return (centre == pt.centre)
              && (rapport.confonduAvec(pt.rapport));
    } else return false;
  }

  public double transformeLongueur(double L) {
    return L*Math.abs(valeurRapport);
  }

  public void transformePoint(double x, double y, Point2D.Double pointr) {
    pointr.x = centre.x + valeurRapport*(x - centre.x);
    pointr.y = centre.y + valeurRapport*(y - centre.y);
  }

  public void transformeVecteur(VecteurDirecteur u, VecteurDirecteur ur) {
    ur.x = valeurRapport*u.x;
    ur.y = valeurRapport*u.y;
  }

  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (centre == ancienPoint) centre = nouveauPoint;
  }
  
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    centre = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    rapport = new CValeurAssocieeAVariable();
    rapport.chargeDepuisFlux(inps, list);
  }  
  /* Enregistrement dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    centre = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    rapport = new CValeurAssocieeAVariable();
    rapport.read(inps, list);
  }  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(centre);
    oups.writeInt(ind1);
    rapport.write(oups, list);
  }   
}