/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.geom.*;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

public class CIntDroiteCercle extends CBipoint {
  private CDroiteAncetre D;
  private CCercle C;
  private Point2D.Double pointInt1 = new Point2D.Double();
  private Point2D.Double pointInt2 = new Point2D.Double();
  
  /** Creates new CIntDroiteCercle */
  public CIntDroiteCercle() {
    super(); // Ncessaire pour crer les deux points CPOintDansBipoint maintenus dans l'objet
  }
  
  // Constructeur pour clonage
  public CIntDroiteCercle(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    CDroiteAncetre DInit, CCercle CInit) {
    super(liste, impProtoInit, estElementFinalInit); // Ncessaire pour crer les deux points CPOintDansBipoint maintenus dans l'objet
    D = DInit;
    C =CInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(D);
    int ind2 = listeSource.indexOf(C);
    int ind3 = listeSource.indexOf(impProto);
    CElementBase ptelb = new CIntDroiteCercle(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, (CDroiteAncetre)(listeCible.get(ind1, CDroiteAncetre.class)), 
      (CCercle)(listeCible.get(ind2, CCercle.class)));
    return ptelb;
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo38");
  }

  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    liste.add(D);
    liste.add(C);
  } 
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || D.dependDe(p) || C.dependDe(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || D.dependDePourBoucle(p) || C.dependDePourBoucle(p));
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    byte position;
    
    existe = C.existe && D.existe;
    if  (!existe) {
      point1.existe = false;
      point2.existe = false;
      return;
    }
    position = Kernel.intersectionDroiteCercle(D.point_x,D.point_y,D.vecteur,
      C.centre_x,C.centre_y,C.rayon,pointInt1,pointInt2);
    double x1 = pointInt1.x;
    double y1 = pointInt1.y;
    double x2 = pointInt2.x;
    double y2 = pointInt2.y;
    
    switch (position) {
      case PositionDroiteCercle.Vide:
        point1.existe = false;
        point2.existe = false;
        break;
      case PositionDroiteCercle.Secants:
        if (D.appartientA(x1,y1) && C.surArc(x1,y1)) {
          point1.existe = true;
          point1.placeEn(x1,y1);
          point1.positionne(infoRandom, dimfen); // Ajout version 4.6.4
        } else point1.existe = false;
        if (D.appartientA(x2,y2) && C.surArc(x2,y2)) {
          point2.existe = true;
          point2.placeEn(x2,y2);
          point2.positionne(infoRandom, dimfen); // Ajout version 4.6.4
        } else point2.existe = false;
        break;
      case PositionDroiteCercle.Tangents:
        if (D.appartientA(x1,y1) & C.surArc(x1,y1)) {
          point1.existe = true;
          point1.placeEn(x1,y1);
          point1.positionne(infoRandom, dimfen); // Ajout version 4.6.4
        } else point1.existe = false;
        point2.existe = false;
        break;
    } // switch
    super.positionne(infoRandom, dimfen);
  }
  
  @Override
  public CElementLigne elementAssocie(byte indice) {
    if (indice == 0) return D;
    else return C;
  }
  
 

  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
   **/
  @Override
  public boolean confonduAvec(CElementBase p) {
    CIntDroiteCercle pt;
    if (p.getClass() == this.getClass()) {
      pt = (CIntDroiteCercle) p;
      return ((D==pt.D) && (C==pt.C));
    } else return false;
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    D = (CDroiteAncetre) (list.get(ind1,CDroiteAncetre.class));
    C = (CCercle) (list.get(ind2,CCercle.class));
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    D = (CDroiteAncetre) (list.get(ind1,CDroiteAncetre.class));
    C = (CCercle) (list.get(ind2,CCercle.class));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(D);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(C);
    oups.writeInt(ind2);
  }
  
  @Override
  boolean estDefiniPar(CDroiteAncetre droite)
  {
    return droite == D;
  }
  
  @Override
  boolean estDefiniPar(CCercle cercle)
  {
    return cercle == C;
  }
}
