/*
 * CIntegrale.java
 *
 * Created on 22 juillet 2004, 14:10
 */

/**
 *
 * @author  Yves
 * @version
 */
package objets;

// Classe de calcul d'une valeur approche d'intgrale
// d'une fonction f par la mthode de Simson sur un intervalle
// [a; b] o f est une fonction dj dfinie
import java.io.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CIntegrale extends CCalculAncetre {
	public static final int valeurMaxiDen = 200;
  public CFonc fonctionAssociee;
  // Les bornes de l'intervalle d'intgration
  public CValeurAssocieeAVariable a;
  public CValeurAssocieeAVariable b;
  // Modification par rapport  la version C++ : la valeur de n devient dynamique
  // int n ; // Le nombre de subdivisions de l'intervalle sera
  public CValeurAssocieeAVariable n;
  // La valeur de la solution de l'quation 2 n.
  private double valeurIntegrale;
  
  /** Creates new CIntegrale */
  public CIntegrale() {
  }
  
  // Constructeur pour clonage
  public CIntegrale(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    String nomInit, CFonc fonctionAssocieeInit,
    CValeurAssocieeAVariable aInit, CValeurAssocieeAVariable bInit, CValeurAssocieeAVariable nInit) {
    super(liste, impProtoInit, estElementFinalInit, nomInit);
    fonctionAssociee = fonctionAssocieeInit;
    a = aInit;
    b = bInit;
    n = nInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(fonctionAssociee);
    int ind2 = listeSource.indexOf(impProto);
    CValeurAssocieeAVariable aClone = (CValeurAssocieeAVariable) a.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable bClone = (CValeurAssocieeAVariable) b.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable nClone = (CValeurAssocieeAVariable) n.getClone(listeSource, listeCible);
    CElementBase ptelb = new CIntegrale(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CFonc)(listeCible.get(ind1, CFonc.class)), aClone,
      bClone, nClone);
    return ptelb;
  }
 
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NIntegrale;
  }
    
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || a.dependDe(p) || b.dependDe(p) || n.dependDe(p) || fonctionAssociee.dependDe(p);
  }
  
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || a.dependDePourBoucle(p) || b.dependDePourBoucle(p)
    || n.dependDePourBoucle(p) || fonctionAssociee.dependDePourBoucle(p);
  }
  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double bornea,borneb;
    double s1,s2,s;
    double t1,t2,pas;
    int k;
    a.positionne(infoRandom, dimfen);
    b.positionne(infoRandom, dimfen);
    n.positionne(infoRandom, dimfen);
    existe = a.existe() && b.existe() && n.existe();
    if (!existe) return;
    bornea = a.rendValeur(infoRandom);
    borneb = b.rendValeur(infoRandom);
    if (bornea == borneb) {
      existe = true;
      valeurIntegrale = 0;
      return;
    }
    double dnbt = Math.floor(n.rendValeur(false) + 0.5);
    existe = (dnbt >= 1) && (dnbt <= valeurMaxiDen);
    if (!existe) return;
    int n = (int) dnbt;
    pas = (borneb - bornea) / n;
    try {
      s = fonctionAssociee.rendValeurFonction(infoRandom, bornea)+
        fonctionAssociee.rendValeurFonction(infoRandom, borneb);
    } catch (ErreurCalculException err) {
      existe = false;
      return;
    }
    t1 = bornea;
    t2 = bornea - pas/2;
    s1 = 0;
    s2 = 0;
    try {
      for (k = 1; k < n; k++) {
        t1 = t1 + pas;
        s1 = s1 + fonctionAssociee.rendValeurFonction(infoRandom, t1);
      }
      for (k = 1; k <= n; k++) {
        t2 = t2 + pas;
        s2 = s2 + fonctionAssociee.rendValeurFonction(infoRandom, t2);
      }
    } catch (ErreurCalculException err) {
      existe = false;
      return;
    }
    valeurIntegrale = (borneb - bornea)*(s + 2*s1 + 4*s2)/(6*n);
    existe = true;
    return;
  }
  
  public double rendValeur(boolean infoRandom) {
    return valeurIntegrale;
  }

  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    fonctionAssociee = (CFonc) (list.get(ind1,CFonc.class));
    a = new CValeurAssocieeAVariable();
    a.chargeDepuisFlux(inps, list);
    b = new CValeurAssocieeAVariable();
    b.chargeDepuisFlux(inps, list);
    // Dans la version C++, la valeur de n n'tait pas dynamique
    int n1 = inps.readCInt();
    n = new CValeurAssocieeAVariable(list, n1);
  }
  /* Lecture depuis un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    fonctionAssociee = (CFonc) (list.get(ind1,CFonc.class));
    a = new CValeurAssocieeAVariable();
    a.read(inps, list);
    b = new CValeurAssocieeAVariable();
    b.read(inps, list);
    n = new CValeurAssocieeAVariable();
    n.read(inps, list);
  }  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(fonctionAssociee);
    oups.writeInt(ind1);
    a.write(oups, list);
    b.write(oups, list);
    n.write(oups, list);
  }
  
  @Override
	public boolean estConstant() {
		return a.estConstant() && b.estConstant() && n.estConstant() && fonctionAssociee.estConstant();
	}
  
}
