/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 *
 * @author  Yves
 * @version 
 */
package objets;

import java.io.*;
import java.awt.Color;
import Applet.AppletMathGraph32;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CMacroClignotement extends CMacroAvecListe {
	public int dureeAnimation; // Dure en secondes (0 si animation
											// jusqu' clic de souris)
	boolean nombrePairClignotements;

  /** Creates new CMacroClignotement */
  public CMacroClignotement() {
  }
  
  // Constructeur pour clonage
  public CMacroClignotement(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, double x_nomInit, double y_nomInit, double dec_xInit, double dec_yInit, boolean masqueInit,
      CPointAncetre pointLieInit, int indiceTaillePoliceInit, int encadrementInit,
      boolean effacementFondInit, Color couleurFondInit, String intituleInit, String commentaireMacroInit,
      CSousListeObjets listeAssocieeInit, int dureeAnimationInit, int alignementHorizontalInit, int alignementVerticalInit) {
        super (liste, impProtoInit, estElementFinalInit, couleurInit, x_nomInit, y_nomInit,
          dec_xInit, dec_yInit, masqueInit, pointLieInit, indiceTaillePoliceInit, encadrementInit,
          effacementFondInit, couleurFondInit, intituleInit, commentaireMacroInit, listeAssocieeInit,
          alignementHorizontalInit, alignementVerticalInit);
        dureeAnimation = dureeAnimationInit;      
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(impProto);        
    CSousListeObjets listeAssocieeClone = new CSousListeObjets();
    listeAssocieeClone.setImage(listeAssociee, listeSource, listeCible);
    CElementBase ptelb = new CMacroClignotement(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, couleur, x_nom, y_nom, dec_x, dec_y, 
      masque, pointLie, indiceTaillePolice, encadrement,
      effacementFond, couleurFondAffichage, intitule, commentaireMacro, listeAssocieeClone, dureeAnimation,
      alignementHorizontal, alignementVertical);
    return ptelb;
   } 


  @Override
  public int typeAnimation() {
    return CMacro.animationParTimer;
  }

  @Override
  public int frequenceAnimation() {
    return 500; // Frquence de clignotement de une demi seconde
  }
  
  @Override
  public long dureePourAnimation() {
    return dureeAnimation;
  }
  
  @Override
  public boolean arretParClic() {
  	return (dureeAnimation == 0);
  }
  
  @Override
  public void actionAvantExecution(AppletMathGraph32 applet) {
  	super.actionAvantExecution(applet);  // Met executioEnCours  true
    nombrePairClignotements = true;
    timeDebut = System.currentTimeMillis();
    if (dureeAnimation != 0) {
      timeFin = timeDebut + dureeAnimation*100; // La dure est en diximes de secondes
    }
  }
   
  @Override
  // Modifi version 2.0
  public void actionDansPaint(AppletMathGraph32 applet) {
  	applet.paneFigure.effacegBuffer();
  	if (nombrePairClignotements) applet.paneFigure.afficheToutDansgbuffer();
  	else {
  		applet.paneFigure.afficheToutDansgbufferSaufElementsDe(listeAssociee);
  	}
  }

  @Override
  public void executePourTimer(AppletMathGraph32 applet, boolean repaint) {
    if (!executionEnCours) return;
    //TRES IMPORTANT : c'est ici que doit se trouver cette instruction et non dans actionDansPaint() car
    // si deux fentres comportant des clignotements se recouvrent, le prochain repaint se fera pour
    // la zne de recouvrement et le clignotement ne se fait que dans la zne de recouvrement
    nombrePairClignotements = !nombrePairClignotements;
    if (repaint) applet.paneFigure.repaint();
  }
  
  @Override
  public void termineAction(AppletMathGraph32 applet) {
    super.termineAction(applet);
    applet.repaintFigureAvecCalcul();
    if (!nombrePairClignotements) {
      listeAssociee.montreTout(true);
      applet.paneFigure.afficheToutDansgbuffer();
      applet.paneFigure.repaint();
    }
  }

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    dureeAnimation = inps.readCInt();
  }
  /* Lecture depuis dans un flux java */  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    dureeAnimation = inps.readInt(); 
  }  
  /* Enregistrement dans un flux java */
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeInt(dureeAnimation);
  }
}
