/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.*;
import java.awt.geom.Point2D;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

public class CMarqueAngleGeometrique extends CMarqueAngleAncetre {
  boolean angledroit; // true si l'angle est droit
  CPointAncetre A;
  CPointAncetre O;
  CPointAncetre B;
  
  /** Creates new CMarqueAngleGeometrique */
  public CMarqueAngleGeometrique() {
  }
  
  /** Constructeur  pour clonage */
  public CMarqueAngleGeometrique(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit, int styleMarqueInit, double rayonInit, CPointAncetre AInit, CPointAncetre OInit,
    CPointAncetre BInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit, styleMarqueInit, rayonInit);
    A = AInit;
    O = OInit;
    B = BInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(A);
    int ind2 = listeSource.indexOf(O);
    int ind3 = listeSource.indexOf(B);
    int ind4 = listeSource.indexOf(impProto);
    CElementBase ptelb =  new CMarqueAngleGeometrique(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, styleMarque, rayon, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)),(CPointAncetre)(listeCible.get(ind3, CPointAncetre.class)));
    return ptelb;
  }

    
 
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    // return memoriseDependance(super.dependDe(p) | A.dependDe(p) || O.dependDe(p) || B.dependDe(p)); // Corrig version 4.7.6
  	return memoriseDependance(super.dependDe(p) || A.dependDe(p) || O.dependDe(p) || B.dependDe(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    // return ((p == this) | A.dependDePourBoucle(p) || O.dependDePourBoucle(p) // Corrig version 4.7.6
  	return ((p == this) || A.dependDePourBoucle(p) || O.dependDePourBoucle(p) || B.dependDePourBoucle(p));
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = (O.existe) && (A.existe) && (B.existe) && !(O.presqueEgal(A)) && !(O.presqueEgal(B));
    if (!(existe)) return;
    centre_x = O.x;
    centre_y = O.y;
    u.setVecteur(O,A);
    v.setVecteur(O,B);
    ang1rad = u.angleRad();
    ang2rad = v.angleRad();
    if (Kernel.zeroAngle(ang1rad - ang2rad)) {
      existe = false;
      return;
    }
    if (Kernel.testAngleDroit(ang1rad,ang2rad)) {
      prepareAngleDroit();
      point2.positionne(infoRandom, dimfen);
      pointc.positionne(infoRandom, dimfen);
      segment1.positionne(infoRandom, dimfen);
    	marqueSegment.positionne(false, dimfen);      
      angledroit = true;
    } else angledroit = false;
    super.positionne(infoRandom, dimfen);
  }
  
  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
   	
    if (!(existe) || (masquage && masque))
    return -1;
    else {
    	boolean appsecteur;
    	VecteurDirecteur w = new VecteurDirecteur(centre_x, centre_y, xp, yp );
			double ang = w.angleRad();
			double ang1radaux = Math.min(ang1rad, ang2rad);
			double ang2radaux = Math.max(ang1rad, ang2rad);
			double dif = ang2radaux - ang1radaux;
			if (Kernel.zeroAngle(dif-Math.PI))
			{
				if (ang1rad < ang2rad) appsecteur = (ang>=ang1rad) && (ang<=ang2rad);
				else appsecteur = (ang >= ang1rad) || (ang <= ang2rad);
			}
			else
			{
				if (dif <= Math.PI) appsecteur = (ang>=ang1radaux) && (ang<ang2radaux);
				else appsecteur = (ang<=ang1radaux) || (ang>=ang2radaux);
			}
			if (!(appsecteur)) return -1;
			if (Kernel.testAngleDroit(ang1rad,ang2rad))
			{
					Point2D.Double proj = new Point2D.Double();
					Kernel.projetteOrtho(xp,yp,pointc.x,pointc.y,u,proj);
					if (Kernel.appartientSegment(proj.x,proj.y,point1.x,point1.y,pointc.x,pointc.y))
						return Kernel.distancePointPoint(xp,yp,proj.x,proj.y);
					else
					{
							Kernel.projetteOrtho(xp,yp,pointc.x,pointc.y,v,proj);
							if (Kernel.appartientSegment(proj.x,proj.y,point2.x,point2.y,pointc.x,pointc.y))
								return Kernel.distancePointPoint(xp,yp,proj.x,proj.y);
							else return -1;
					}
			}
			else return Math.abs(Kernel.distancePointPoint(centre_x,centre_y,xp,yp) - rayon);
    }
  }
  
  /**
   * Remplissage du secteur pour les marques pleines
   * A redfinier pour les marques d'angles directes et indirectes
   * @param pDC
   * @param ang
   */
  protected void remplitSecteur(Graphics2D pDC, double ang, float transparence) {
    Composite originalComposite = pDC.getComposite();
    pDC.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, transparence));
    Kernel.secteurSaillantPlein(pDC, A.x, A.y, O.x, O.y, rayon, ang);	  	
    pDC.setComposite(originalComposite);    		
  }
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    
    if (A.horsEcran || O.horsEcran || B.horsEcran) return;
    int styleTrait;
    if (masque) styleTrait = StyleTrait.traitPointille;
    else styleTrait = style;
    pDC.setStroke(Pinceau.CreePinceau(styleTrait));
    pDC.setColor(couleur);
    if (angledroit) {
      traceAngleDroit(pDC);
      if ((styleMarque & StyleMarqueAngle.marqueRemplie) != 0) {
  	    Composite originalComposite = pDC.getComposite();
  	    pDC.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, transparence));
  	    Kernel.traceQuadrilaterePlein(pDC, centre_x, centre_y, point1.x, point1.y,
  	    		pointc.x, pointc.y, point2.x, point2.y);
  	    pDC.setComposite(originalComposite);
      }
	    if ((styleMarque & StyleMarqueAngle.marqueAvecTrait) != 0) {
	    	marqueSegment.donneCouleur(couleur);
	    	marqueSegment.donneStyle(styleTrait);
	    	marqueSegment.donneMotif(StyleMarqueAngle.styleMarqueSegment(styleMarque));
	    	marqueSegment.trace(pDC, couleurFond, transparence);
	    }
    } else {
      double mesureAngle = u.mesureAngleVecteurs(v);
      if (Kernel.zeroAngle(Math.PI - Math.abs(mesureAngle))) mesureAngle = Math.PI;
      Kernel.secteurSaillant(pDC,A.x,A.y,O.x,O.y,rayon,mesureAngle);
      if ((styleMarque & StyleMarqueAngle.marqueRemplie) != 0) remplitSecteur(pDC, mesureAngle, transparence);	
      if ((styleMarque & StyleMarqueAngle.marqueAvecTrait) != 0) traceMarqueAngle(pDC, A.x, A.y, O.x, O.y, mesureAngle);
    }
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    int ind3 = inps.readCInt();
    A = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    O = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    B = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
  }

  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (A == ancienPoint) A = nouveauPoint;
    if (B == ancienPoint) B = nouveauPoint;
    if (O == ancienPoint) O = nouveauPoint;
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    int ind3 = inps.readInt();
    A = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    O = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    B = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(A);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(O);
    oups.writeInt(ind2);
    int ind3 = list.indexOf(B);
    oups.writeInt(ind3);
  }  
 }
