/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 *
 * @author  Yves
 * @version 
 */
package objets;

import types.*;
import glob.*;


public class CMesureAngleOriente extends CMesureAngleGeometrique {

  /** Creates new CMesureAngleOriente */
  public CMesureAngleOriente() {
  }
  
  // Constructeur pour clonage
  public CMesureAngleOriente(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CPointAncetre point1Init, CPointAncetre point2Init, CPointAncetre point3Init) {
    super(liste, impProtoInit, estElementFinalInit, point1Init, point2Init, point3Init);
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(point1);
    int ind2 = listeSource.indexOf(point2);   
    int ind3 = listeSource.indexOf(point3);  
    int ind4 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMesureAngleOriente(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind3, CPointAncetre.class)));
    return ptelb;    
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NMesureAngleOriente;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch =  getString("chinfo30") + " " + nom();
    if (existe) ch = ch + "\n" + getString("chinfo97") + " " + Kernel.chaineNombre(mesure, 16);
    else ch = ch + "\n" + getString("ch18");
    return ch;        
  }
  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double prosca,deter,tangente,resul;
    
    existe = point1.existe && point2.existe && point3.existe
			&& !(point2.egal(point1)) && !(point2.egal(point3));
    if (!existe) return;
    // L'orientation sur l'cran est l'inverse de celle du repre
		// choisi.Il faut donc permuter les rles de U et V
    VecteurDirecteur v = new VecteurDirecteur(point2,point1);
    VecteurDirecteur u = new VecteurDirecteur(point2,point3);
    prosca = u.x*v.x + u.y*v.y;
    deter = u.x*v.y - u.y*v.x;
    if (prosca == 0)
		{
			if (deter > 0)  resul = Math.PI/2;
			else resul = - Math.PI/2;
		}
    else
    {
      tangente = deter/prosca;
      if (prosca > 0) resul = Math.atan(tangente);
      else
      {
        if (deter >= 0) resul = Math.atan(tangente) + Math.PI;
        else resul = Math.atan(tangente) - Math.PI;
      }
    }
    // Nouveau pour version 1.3.6 : on regarde si l'angle est trs proche de - pi. Si oui on renvoie pi
    if (Math.abs(resul + Math.PI)< 1e-13)resul = Math.PI;
    if (listeProprietaire.uniteAngle == UniteAngle.Radian) mesure = resul;
    else mesure = resul*Kernel.ConvRadDeg;
	}
  
  @Override  
  public String nom() {
    return "(" + point2.nom + point1.nom + "," + point2.nom + point3.nom + ')';
  }
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  public boolean confonduAvec(CElementBase p) {
    CMesureAngleOriente pt;
    if (p.getClass() == this.getClass()) {
      pt = (CMesureAngleOriente) p;
      return ((point2 == pt.point2) && 
			(point1 == pt.point1) && (point3 == pt.point3));
    }
    else return false;
  }    
}
