/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.IOException;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.DimensionFenetre;
import types.NatureCalcul;

public class CMesureCoefDir extends CCalculAncetre {
	double coef;
	CRepere rep; // Le repre dans lequel se fait la mesure
	CDroite d; // La droite dont on mesure le coeffcient directeur

  /** Constructeur */
  public CMesureCoefDir() {
  }
  
  /** Constructeur pour clonage */
  public CMesureCoefDir(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CDroite dInit, CRepere repInit, String nomInit) {
    super(liste, impProtoInit, estElementFinalInit, nomInit);
    d = dInit;
    rep = repInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(d);
    int ind2 = listeSource.indexOf(rep);   
    int ind3 = listeSource.indexOf(impProto);    
    CElementBase ptelb = new CMesureCoefDir(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, (CDroite)(listeCible.get(ind1, CDroite.class)),
      (CRepere)(listeCible.get(ind2, CRepere.class)), nomCalcul);
    return ptelb;    
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NMesureCoefDir;
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || rep.dependDe(p) || d.dependDe(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || rep.dependDePourBoucle(p) || d.dependDePourBoucle(p);
  }  
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = rep.existe && d.existe;
    if (!existe) return;
    // On calcule les coordonnes du vecteur directeur de la droite dans le repre  un coeff prs
    double a = d.vecteur.x; // Premire coordonne du vecteur directeur de la droite
    double b = d.vecteur.y; // Deuxime coordonne
    double x = rep.v.y*a-rep.v.x*b;
    double y = rep.u.x*b-rep.u.y*a;
    if (x == 0) {
    	existe = false;
    	return;
    }
    // Modification version 4.0 car les repres ont une unit sur chaque axe
    // coef = y/x;
    coef = y/x/rep.unitex*rep.unitey;
  }  
	
  @Override
  public double rendValeur(boolean infoRandom) {
    return coef;
  }
    
  public boolean confonduAvec(CElementBase p) {
    CMesureCoefDir pt;
    if (p.getClass() == this.getClass()) {
      pt = (CMesureCoefDir) p;
      return (rep == pt.rep) && (d == pt.d);
    }
    else return false;
  }    
  
  @Override
  public boolean nomIndispensable(CElementGraphique el) {
    return (el == d);
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    rep = (CRepere) (list.get(ind1,CRepere.class));
    d = (CDroite) (list.get(ind2,CDroite.class));
 }
  

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
   super.read(inps, list);
   int ind1 = inps.readInt();
   int ind2 = inps.readInt();
   rep = (CRepere) (list.get(ind1,CRepere.class));
   d = (CDroite) (list.get(ind2,CDroite.class));
 }  
 

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
   super.write(oups, list);
   int ind1 = list.indexOf(rep);
   oups.writeInt(ind1);
   int ind2 = list.indexOf(d);
   oups.writeInt(ind2);
 }     
  
}
