/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.util.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

/**
 *
 * @author Yves
 */
public class CMesureLongueurLigne extends CCalculAncetre {
	private CLignePolygonaleAncetre ligneAssociee;
  private double longueur;
  private VecteurDirecteur u = new VecteurDirecteur(); // Pour le positionnement
  
  /** Creates a new instance of CMesureLongueurLigne */
  public CMesureLongueurLigne() {
  }

  // Constructeur pour clonage
  public CMesureLongueurLigne(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CLignePolygonaleAncetre ligneAssocieeInit, String nomCalculInit) {
    // Inutile d'appeler le constructeur anctre vide
    super(liste, impProtoInit, estElementFinalInit, nomCalculInit);
    ligneAssociee = ligneAssocieeInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(ligneAssociee);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb =  new CMesureLongueurLigne(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, (CLignePolygonaleAncetre)(listeCible.get(ind1, CLignePolygonaleAncetre.class)), nomCalcul);
    return ptelb;
  }
  
  @Override
  public boolean nomIndispensable(CElementGraphique el) {
  	for (int i =0; i < ligneAssociee.collectionPoints.size(); i++) {
  		CNoeudPointeurSurPoint nd = (CNoeudPointeurSurPoint) ligneAssociee.collectionPoints.get(i);
  		if (nd.pointeurSurPoint == el) return true;
  	}
  	return false;
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NMesureLongueurLigne;
  }
  
  @Override
  public void setClone(CElementBase ptel) {
    longueur = ((CMesureLongueurLigne) ptel).longueur;
  }
  
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || ligneAssociee.dependDe(p)
    || listeProprietaire.pointeurLongueurUnite.dependDe(p));
  }
    
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || ligneAssociee.dependDePourBoucle(p)
    || listeProprietaire.pointeurLongueurUnite.dependDePourBoucle(p);
  }

  public double rendValeur(boolean infoRandom) {
    return longueur;
  }

  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    CLongueur pLongUnite = listeProprietaire.pointeurLongueurUnite;
		existe = ligneAssociee.existe && pLongUnite.existe;
    if (!existe) return;
    double longueurUnite = pLongUnite.rendLongueur();
    // Si la longueur unit est nulle, pLongUnite n'existe pas
    longueur = 0;
    Vector<CNoeudPointeurSurPoint> array = ligneAssociee.collectionPoints;
    for (int i = 0; i < array.size() - 1; i++ ) {
      CPointAncetre ptp1 = (CPointAncetre)((CNoeudPointeurSurPoint) array.get((i))).pointeurSurPoint;
      CPointAncetre ptp2 = (CPointAncetre)((CNoeudPointeurSurPoint) array.get((i+1))).pointeurSurPoint;
      u.x = ptp2.x - ptp1.x;
      u.y = ptp2.y - ptp1.y;
      longueur += u.norme();
    }
    longueur /= longueurUnite;
  }



  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  public boolean confonduAvec(CElementBase p) {
    CMesureLongueurLigne pt;
    if (p.getClass() == this.getClass()) {
      pt = (CMesureLongueurLigne) p;
      return ligneAssociee == pt.ligneAssociee;
    }
    else return false;
  }
  
  @Override
  public boolean utiliseLongueurUnite() {
  	return true;
  }
  
        
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    ligneAssociee = (CLignePolygonaleAncetre) (list.get(ind1,CLignePolygonaleAncetre.class));
  }
  /* Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    ligneAssociee = (CLignePolygonaleAncetre) (list.get(ind1,CLignePolygonaleAncetre.class));
  }  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(ligneAssociee);
    oups.writeInt(ind1);
  } 
}
