/*
 * CMesureX.java
 *
 * Created on 12 juillet 2004, 20:36
 */

/**
 *
 * @author  Yves
 * @version 
 */
package objets;

import java.io.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.Kernel;

public class CMesureX extends CValeurDynamique {
	double abscisse;
	CRepere repereAssocie;
	CPointAncetre pointMesure;

  /** Creates new CMesureX */
  public CMesureX() {
  }
  
  // Constructeur pour clonage
  public CMesureX(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CRepere repereAssocieInit, CPointAncetre pointMesureInit) {
    super(liste, impProtoInit, estElementFinalInit);
    repereAssocie = repereAssocieInit;
    pointMesure = pointMesureInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(repereAssocie);
    int ind2 = listeSource.indexOf(pointMesure); 
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMesureX(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, (CRepere)(listeCible.get(ind1, CRepere.class)), (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)));
    return ptelb;    
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NMesureAbscisseRepere;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch = textes.Chaines.chinfo31 + " " + pointMesure.rendNom() + "\n" + textes.Chaines.chinfo32 + " "
    + repereAssocie.nom() + "\n"
    + textes.Chaines.chinfo33 + " " + nom();
    if (existe) ch = ch + "\n"	+ textes.Chaines.chinfo97 + Kernel.chaineNombre(abscisse, 16);
    else ch = ch + "\n" + textes.Chaines.ch18;
    return ch;
  }
  
  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    liste.add(pointMesure);
  } 
  
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || repereAssocie.dependDe(p) || pointMesure.dependDe(p);
  }

  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || repereAssocie.dependDePourBoucle(p) || pointMesure.dependDePourBoucle(p);
  }

  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = repereAssocie.existe && pointMesure.existe;
    if (!existe) return;
    // L'orientation sur l'cran est l'inverse de celle du repre
		// choisi.Il faut donc permuter les rles de u et v}
    VecteurDirecteur u = new VecteurDirecteur(repereAssocie.O,repereAssocie.I);
    VecteurDirecteur v = new VecteurDirecteur(repereAssocie.O,repereAssocie.J);
    // Calcul des	abscisses et ordonnes de l'origine relle du repre dans le repre de base
    double xno = repereAssocie.O.x - u.x*repereAssocie.abscisseOrigine.rendValeur(infoRandom)-
            v.x*repereAssocie.ordonneeOrigine.rendValeur(infoRandom);
    double yno = repereAssocie.O.y - u.y*repereAssocie.abscisseOrigine.rendValeur(infoRandom)-
            v.y*repereAssocie.ordonneeOrigine.rendValeur(infoRandom);
    // Calcul de l'abscisse par rsolution d'un systme linaire
    double det = u.x*v.y-v.x*u.y;
    abscisse = (v.y*(pointMesure.x-xno)-v.x*(pointMesure.y-yno))/det;
  }

  @Override
  public double rendValeur(boolean infoRandom) {
    return abscisse;
  }
    
  @Override
  public String nom() {
    return "xCoord" + "(" + pointMesure.nom + "," + repereAssocie.O.nom + "," +
      repereAssocie.I.nom + "," +repereAssocie.J.nom + ")";
  }

  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (pointMesure == ancienPoint) pointMesure = nouveauPoint;
  }
  
  public boolean confonduAvec(CElementBase p) {
    CMesureX pt;
    if (p.getClass() == this.getClass()) {
      pt = (CMesureX) p;
      return (repereAssocie == pt.repereAssocie) && (pointMesure == pt.pointMesure);
    }
    else return false;
  }    
  
  @Override
  public boolean nomIndispensable(CElementGraphique el) {
    return (el == pointMesure);
  }  
        
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
     super.chargeDepuisFlux(inps, list);
     int ind1 = inps.readCInt();
     int ind2 = inps.readCInt();
     repereAssocie = (CRepere) (list.get(ind1,CRepere.class));
     pointMesure = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }
  /* Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    repereAssocie = (CRepere) (list.get(ind1,CRepere.class));
    pointMesure = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(repereAssocie);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(pointMesure);
    oups.writeInt(ind2);
  }   
}
