/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 *
 * @author  Yves
 * @version
 */
package objets;

import java.io.*;
import java.awt.Color;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CMilieu extends CPointAncetre {
  CPointAncetre point1;
  CPointAncetre point2; //Pointeur sur les deux extrmits
  
  /** Creates new CMilieu */
  public CMilieu() {
  }
  public CMilieu(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
    boolean masqueInit, String nomInit, int indiceTailleNomInit,
    int motifInit , CPointAncetre point1Init, CPointAncetre point2Init) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
      dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit, false);
    motif = motifInit;
    point1 = point1Init;
    point2 = point2Init;
  }
  /* Constructeur plus court pour objets visuels */
  public CMilieu(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, int motifInit , CPointAncetre point1Init, CPointAncetre point2Init) {
    super(liste,  impProtoInit, estElementFinalInit, couleurInit,
      false, 0d, 0d,  false,  "", 2,  MotifPoint.petitRond, false);
    motif = motifInit;
    point1 = point1Init;
    point2 = point2Init;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(point1);
    int ind2 = listeSource.indexOf(point2);
    int ind3 = listeSource.indexOf(impProto);
    CElementBase ptelb = new CMilieu(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom, indiceTailleNom, motif,
      (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),(CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)));
    return ptelb;
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo8");
  }
  

  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    liste.add(point1);
    liste.add(point2);
  }  
  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = ((point1.existe) && (point2.existe));
    if (existe) {
      super.placeEn((point1.x + point2.x)/2, (point1.y + point2.y)/2);
      super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
    }
  }
  
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || point1.dependDe(p) || point2.dependDe(p));
  }
  
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || point1.dependDePourBoucle(p) || point2.dependDePourBoucle(p);
  }
  
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
   **/
  public boolean confonduAvec(CElementBase p) {
    CMilieu pt;
    if (p.getClass() == this.getClass()) {
      pt = (CMilieu) p;
      return (((point1==pt.point1) && (point2==pt.point2))
      || ((point2==pt.point1) && (point1==pt.point2)));
    } else return false;
  }
  
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (point1 == ancienPoint) point1 = nouveauPoint;
    if (point2 == ancienPoint) point2 = nouveauPoint;
  }
  
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    // A revoir java
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    point1 = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    point2 = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }
  /* Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    point1 = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    point2 = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(point1);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(point2);
    oups.writeInt(ind2);
  }
  
  @Override
  public boolean appartientParDefinition(CDroiteAncetre droite) {
  	return point1.appartientParDefinition(droite) && point2.appartientParDefinition(droite);
  }
  
}
