/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;


/**
 *
 * @author Yves
 */
public class CPartieReelle extends CCalculAncetre {
  public CValeurDynamique complexeAssocie;
  // Attribut ajout pour optimisation
  Complexe z1 = new Complexe();
  double resultat;
  /** Creates a new instance of CPartieReelle */
  public CPartieReelle() {
  }
  // Constructeur pour clonage
  public CPartieReelle(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      String nomCalculInit, CValeurDynamique complexeAssocieInit) {
    super(liste, impProtoInit, estElementFinalInit, nomCalculInit);
    complexeAssocie = complexeAssocieInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(complexeAssocie);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb =  new CPartieReelle(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CValeurDynamique)(listeCible.get(ind1, CValeurDynamique.class)));
    return ptelb;
  }
 
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NPartieReelle;
  }
  
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || complexeAssocie.dependDe(p));
  }
   
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || complexeAssocie.dependDePourBoucle(p);
  }
  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = complexeAssocie.existe;
    if (!existe) return;
    complexeAssocie.rendValeurComplexe(infoRandom, z1);
    resultat = z1.x;
  }
  public double rendValeur(boolean infoRandom) {
    return resultat;
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CPartieReelle pt;
    if (p.getClass() == this.getClass()) {
      pt = (CPartieReelle) p;
      return complexeAssocie == pt.complexeAssocie;
    }
    else return false;
  }
  
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    complexeAssocie = (CValeurDynamique) (list.get(ind1,CValeurDynamique.class));
  }
  /* Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    complexeAssocie = (CValeurDynamique) (list.get(ind1,CValeurDynamique.class));
  }  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(complexeAssocie);
    oups.writeInt(ind1);
  }
  
  @Override
	public boolean estConstant() {
		return complexeAssocie.estConstant();
	}
  
}
