/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.Color;
import java.io.IOException;

import Applet.AppletMathGraph32;

import types.DimensionFenetre;
import types.NatureObjet;
import types.VecteurDirecteur;

import dialogs.CoordonneesDlg;
import dialogs.MtgDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;


public class CPointDansRepere extends CPointAncetre {
	public CRepere rep;
	public CValeurAssocieeAVariable abs;
	public CValeurAssocieeAVariable ord;

  /** Creates new CPointDansRepere */
  public CPointDansRepere() {
    abs = new CValeurAssocieeAVariable();
    ord = new CValeurAssocieeAVariable();
  }
  
  // Constructeur pour objets internes dans les suites rcurrentes
  public CPointDansRepere(CListeObjets liste, Color couleurInit, int motifInit, CRepere repInit) {
    super (liste, null, false, couleurInit, true, 0d, 0d, false, "", 0, motifInit, false);
    rep = repInit;
    abs = new CValeurAssocieeAVariable();
    ord = new CValeurAssocieeAVariable();   
  }
  
  // Constructeur pour clonage
  public CPointDansRepere(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit,double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit , CRepere repInit, CValeurAssocieeAVariable absInit,
      CValeurAssocieeAVariable ordInit, boolean marquePourTraceInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit, dec_xInit, dec_yInit,
      masqueInit, nomInit, indiceTailleNomInit, motifInit, marquePourTraceInit);
    rep = repInit;
    abs = absInit;
    ord = ordInit;
  }
        
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(rep);
    int ind2 = listeSource.indexOf(impProto);        
    CValeurAssocieeAVariable absClone = (CValeurAssocieeAVariable) abs.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable ordClone = (CValeurAssocieeAVariable) ord.getClone(listeSource, listeCible);
    return new CPointDansRepere(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom,
          indiceTailleNom, motif, (CRepere)(listeCible.get(ind1, CRepere.class)), absClone, ordClone, marquePourTrace);
  }  

  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || rep.dependDe(p) || abs.dependDe(p) || ord.dependDe(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || rep.dependDePourBoucle(p) || abs.dependDePourBoucle(p)
    || ord.dependDePourBoucle(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	abs.positionne(infoRandom, dimfen);
  	ord.positionne(infoRandom, dimfen);
  	existe = abs.existe() && ord.existe() && rep.existe;
    if (!existe) return;
    double k1 = abs.rendValeur(infoRandom);
    double k2 = ord.rendValeur(infoRandom);
		VecteurDirecteur vec = new VecteurDirecteur();
		double unitex = rep.unitex;
		double unitey = rep.unitey;
		
    vec.x = k1*rep.u.x/unitex + k2*rep.v.x/unitey;
    vec.y = k1*rep.u.y/unitex + k2*rep.v.y/unitey;
    x = rep.origine.x + vec.x;
    y = rep.origine.y + vec.y;
    super.positionne(infoRandom, dimfen);
  }

  @Override
  public boolean confonduAvec(CElementBase p) {
    CPointDansRepere pt;
    if (p.getClass() == this.getClass()) {
      pt = (CPointDansRepere) p;
   		return (rep == pt.rep) && abs.confonduAvec(pt.abs) && ord.confonduAvec(pt.ord) ;
    }
    else return false;
  }        
  
  @Override
  public long getNature() {
    return NatureObjet.NPointDansRepere;
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo140") + " (" + abs.chaineInfo() + ";" + ord.chaineInfo() + ")\n" +
    getString("chinfo32") + " " + rep.nom();
  }
  

  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    liste.add(rep.O);
    liste.add(rep.I);
    liste.add(rep.J);
  }    

  @Override
  public boolean modifiableParMenu() {
  	// Modification version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(AppletMathGraph32 applet) {
  	return new CoordonneesDlg(applet, this, true);
  }
  
  @Override
  public String informationModification() {
  	return getString("chaineOutil47");
  }
    
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    rep = (CRepere) (list.get(ind1,CRepere.class));
    abs.chargeDepuisFlux(inps, list);
    ord.chargeDepuisFlux(inps, list);
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    rep = (CRepere) (list.get(ind1,CRepere.class));
    abs.read(inps, list);
    ord.read(inps, list);    
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(rep);
    oups.writeInt(ind1);
    abs.write(oups, list);
    ord.write(oups, list);
  }
}
