/*
 * CPointDansRepere.java
 *
 * Created on 12 juillet 2004, 18:36
 */

/**
 *
 * @author  Yves
 * @version 
 */
package objets;

import java.awt.Color;
import java.io.IOException;
import types.DimensionFenetre;
import types.NatureObjet;
import types.VecteurDirecteur;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CPointDansRepere extends CPointAncetre {
	public CRepere rep;
	public CValeurAssocieeAVariable abs;
	public CValeurAssocieeAVariable ord;

  /** Creates new CPointDansRepere */
  public CPointDansRepere() {
    abs = new CValeurAssocieeAVariable();
    ord = new CValeurAssocieeAVariable();
  }
  
  // Constructeur pour objets internes dans les suites rcurrentes
  public CPointDansRepere(CListeObjets liste, Color couleurInit, int motifInit, CRepere repInit) {
    super (liste, null, false, couleurInit, true, 0d, 0d, false, "", 0, motifInit);
    rep = repInit;
    abs = new CValeurAssocieeAVariable();
    ord = new CValeurAssocieeAVariable();   
  }
  
  // Constructeur pour clonage
  public CPointDansRepere(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit,double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit , CRepere repInit, CValeurAssocieeAVariable absInit,
      CValeurAssocieeAVariable ordInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit, dec_xInit, dec_yInit,
      masqueInit, nomInit, indiceTailleNomInit, motifInit);
    rep = repInit;
    abs = absInit;
    ord = ordInit;
  }
        
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(rep);
    int ind2 = listeSource.indexOf(impProto);        
    CValeurAssocieeAVariable absClone = (CValeurAssocieeAVariable) abs.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable ordClone = (CValeurAssocieeAVariable) ord.getClone(listeSource, listeCible);
    return new CPointDansRepere(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom,
          indiceTailleNom, motif, (CRepere)(listeCible.get(ind1, CRepere.class)), absClone, ordClone);
  }  

  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || rep.dependDe(p) || abs.dependDe(p) || ord.dependDe(p);
  }

  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || rep.dependDePourBoucle(p) || abs.dependDePourBoucle(p)
    || ord.dependDePourBoucle(p));
  }

  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	abs.positionne(infoRandom, dimfen);
  	ord.positionne(infoRandom, dimfen);
  	existe = abs.existe() && ord.existe() && rep.existe;
    if (!existe) return;
    double k1 = abs.rendValeur(infoRandom);
    double k2 = ord.rendValeur(infoRandom);
		VecteurDirecteur vec = new VecteurDirecteur();
    vec.x = k1*rep.u.x + k2*rep.v.x;
    vec.y = k1*rep.u.y + k2*rep.v.y;
    x = rep.origine.x + vec.x;
    y = rep.origine.y + vec.y;
    super.positionne(infoRandom, dimfen);
  }

  @Override
  public boolean confonduAvec(CElementBase p) {
    CPointDansRepere pt;
    if (p.getClass() == this.getClass()) {
      pt = (CPointDansRepere) p;
   		return (rep == pt.rep) && abs.confonduAvec(pt.abs) && ord.confonduAvec(pt.ord) ;
    }
    else return false;
  }        
  
  @Override
  public long getNature() {
    return NatureObjet.NPointDansRepere;
  }
  
  @Override
  public String infoPourHistorique() {
    return textes.Chaines.chinfo140 + " (" + abs.chaineInfo() + ";" + ord.chaineInfo() + ")\n" +
    textes.Chaines.chinfo32 + " " + rep.nom();
  }
  

  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    liste.add(rep.O);
    liste.add(rep.I);
    liste.add(rep.J);
  }    
  
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    rep = (CRepere) (list.get(ind1,CRepere.class));
    abs.chargeDepuisFlux(inps, list);
    ord.chargeDepuisFlux(inps, list);
  }
  /* Chargement depuis un flux java */
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    rep = (CRepere) (list.get(ind1,CRepere.class));
    abs.read(inps, list);
    ord.read(inps, list);    
  }
  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(rep);
    oups.writeInt(ind1);
    abs.write(oups, list);
    ord.write(oups, list);
  }
}
