/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.Color;
import java.io.IOException;
import types.Complexe;
import types.DimensionFenetre;
import types.NatureObjet;
import types.VecteurDirecteur;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

/**
 *
 * @author Yves
 */
public class CPointParAffixe extends CPointAncetre {
  public CRepere rep;
  public CValeurAssocieeComp affixe;
  // Attribut ajout pour optimisation 
  private Complexe z1 = new Complexe();
  
  /** Creates a new instance of CPointParAffixe */
  public CPointParAffixe() {
    affixe = new CValeurAssocieeComp();
  }
  // Constructeur pour clonage
  public CPointParAffixe(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit , CRepere repInit, CValeurAssocieeComp affixeInit, boolean marquePourTraceInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit, dec_xInit, dec_yInit,
      masqueInit, nomInit, indiceTailleNomInit, motifInit, marquePourTraceInit);
    rep = repInit;
    affixe = affixeInit;
  }
  
  @Override  
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(rep);
    int ind2 = listeSource.indexOf(impProto);        
    CValeurAssocieeComp affixeClone = (CValeurAssocieeComp) affixe.getClone(listeSource, listeCible);
    return new CPointParAffixe(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom,
          indiceTailleNom, motif, (CRepere)(listeCible.get(ind1, CRepere.class)), affixeClone, marquePourTrace);
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || rep.dependDe(p) || affixe.dependDe(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || rep.dependDePourBoucle(p) || affixe.dependDePourBoucle(p));
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	affixe.positionne(infoRandom, dimfen);
    existe = affixe.existe() && rep.existe;
    if (!existe) return;
    affixe.rendValeurComplexe(infoRandom, z1);
    VecteurDirecteur vec = new VecteurDirecteur();
		double unitex = rep.unitex;
		double unitey = rep.unitey;
    vec.x = z1.x*rep.u.x/unitex + z1.y*rep.v.x/unitey;
    vec.y = z1.x*rep.u.y/unitex + z1.y*rep.v.y/unitey;
    x = rep.origine.x + vec.x;
    y = rep.origine.y + vec.y;
    super.positionne(infoRandom, dimfen);
  }
  @Override
  public long getNature() {
    return NatureObjet.NPointParAffixe;
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CPointParAffixe pt;
    if (p.getClass() == this.getClass()) {
      pt = (CPointParAffixe) p;
      return (rep == pt.rep) && affixe.confonduAvec(pt.affixe);
    }
    else return false;
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo164") + " " + affixe.chaineInfo() + "\n" +
    getString("chinfo32") + " " + rep.nom();
  }
  

  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    rep = (CRepere) (list.get(ind1,CRepere.class));
    affixe.chargeDepuisFlux(inps, list);
  }  

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    rep = (CRepere) (list.get(ind1,CRepere.class));
    affixe = new CValeurAssocieeComp();
    affixe.read(inps, list);
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(rep);
    oups.writeInt(ind1);
    affixe.write(oups, list);
  }
}
