package objets;

import java.awt.Color;
import java.io.IOException;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.DimensionFenetre;

/**
 * Objet rajout pour la version 1.6
 * Crer un point image d'un autre par la translation dont le vecteur est la somme des
 * deux vecteurs cliqus.
 * @author Yves
 *
 */
public class CPointParSommeVect extends CPointAncetre {
	private CPointAncetre antecedent;
	private CVecteur vecteur1;
	private CVecteur vecteur2;
	
	/**
	 * Constructeur par dfaut
	 */
	public CPointParSommeVect() {
		
	}
	
  /** Constructeur pour clonage */
  public CPointParSommeVect(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit , CPointAncetre antecedentInit, CVecteur vecteur1Init, CVecteur vecteur2Init) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
          dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit);
        antecedent = antecedentInit;
        vecteur1 = vecteur1Init;
        vecteur2 = vecteur2Init;
  }  	

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(antecedent);
    int ind2 = listeSource.indexOf(vecteur1); 
    int ind3 = listeSource.indexOf(vecteur2); 
    int ind4 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CPointParSommeVect(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom,
        indiceTailleNom, motif, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
        (CVecteur)(listeCible.get(ind2)), (CVecteur)(listeCible.get(ind3)));
    return ptelb;
  }
  

  
  @Override
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || antecedent.dependDe(p) || vecteur1.dependDe(p) || vecteur2.dependDe(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || antecedent.dependDePourBoucle(p) || vecteur1.dependDePourBoucle(p)) || vecteur2.dependDePourBoucle(p);
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	
		existe = antecedent.existe && vecteur1.existe && vecteur2.existe;
		if (!existe) return;
		super.placeEn(antecedent.x + vecteur1.xb - vecteur1.xa + vecteur2.xb - vecteur2.xa, 
				antecedent.y + vecteur1.yb - vecteur1.ya + vecteur2.yb - vecteur2.ya);
	}
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CPointParSommeVect pt;
    if (p.getClass() == this.getClass()) {
      pt = (CPointParSommeVect) p;
      return (antecedent.confonduAvec(pt.antecedent) && ((vecteur1.confonduAvec(pt.vecteur1) && vecteur2.confonduAvec(pt.vecteur2))
      		|| (vecteur1.confonduAvec(pt.vecteur2) && vecteur2.confonduAvec(pt.vecteur1))));
    } else return false;
  }
  
  /** Chargement depuis un flux java */
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    int ind3 = inps.readInt();
    antecedent = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    vecteur1 = (CVecteur) (list.get(ind2,CVecteur.class));
    vecteur2 = (CVecteur) (list.get(ind3,CVecteur.class));
  }
  
  /** Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(antecedent);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(vecteur1);
    oups.writeInt(ind2);
    int ind3 = list.indexOf(vecteur2);
    oups.writeInt(ind3);
  }
  
}
