/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/**
 * Cette classe n'est pas pleinement implmente dans l'applet. Elle a seulement besoin d'tre
 * charge.
 **/
package objets;

import java.io.*;
import java.util.ListIterator;
import types.NatureCalcul;
import types.NatureObjet;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
/**
 *
 * @author Yves
 */
 @SuppressWarnings("serial")
public class CPrototype extends CListeObjets {
  // public static final int numeroVersionPrototypes = 2; Version 4.4
	// public static final int numeroVersionPrototypes = 3; Version 4.7.7
	public static final int numeroVersionPrototypes = 4;
	private int numeroVersionPrototype;	// Par prcaution pour de futures versions. Vaudra 1 pour la version 2.5
  public String nom;
  public String commentaire; // Commentaire explicatif
  public int nbObjSources;
  public int nbObjFinaux;
  public int natureSources;
	public CListeObjets listePourImplementation;
  public int indicePremierObjetPourImplementation;
  
	public static int ConstructionEltsGraphiquesSeuls = 0;
	public static int ConstructionEltsMixtes = 1;
	public static int ConstructionEltsCalculsSeuls = 2;
  
  /** Creates a new instance of CPrototype */
  public CPrototype() {
  	numeroVersionPrototype = numeroVersionPrototypes;
  	setListePourImplementation(null);
  }
  
  /**
   * Constructeur  partir d'objets contenus dans la liste listeSources
   * @param nomInit : Le nom du prototype
   * @param nbObjSourcesInit : Le nombre d'objets sources
   * @param listePrincipale : la liste contenant tous les objets dont certains servent  construire le prototype
   * @param listeElements : la sous liste complte de listeElements forme des lments sources, intermdiaires et finaux
   * @param listeElementsFinaux : la sous liste de listeElements forme des lments finaux
   * @param natureSourcesInit : Donne la nature du prototype : sources exclusivement de type calcul ou exclusivement
   * de type graphique ou mixte
   */
  public CPrototype(String nomInit, int natureSourcesInit, int nbObjSourcesInit, CListeObjets listePrincipale, CListeObjets listeElements, 
  		CListeObjets listeElementsFinaux) {
  	nom = nomInit;
  	natureSources = natureSourcesInit;
  	nbObjSources = nbObjSourcesInit;
  	numeroVersionPrototype = numeroVersionPrototypes;
  	setListePourImplementation(null);
  	nbObjFinaux = listeElementsFinaux.size();
  	int nombreTotal = listeElements.size();
  	int i;
  	for (i = 0; i < nbObjSources; i++)
  	{
  		CElementGenerique ptgen = new CElementGenerique();
  		CElementBase elb = listeElements.get(i);
  		long natureGraphique = elb.getNature();
  		long natureCalcul = elb.getNatureCalcul();
  		// Modification version 1.0.1
  		// Modifiv version 3.4.5
  		// if ((natureCalcul & NatureCalcul.NTouteValeurReelleOuComplexe) != 0)
  		if ((natureCalcul & NatureCalcul.NObjetCalculPourSourcesPrototypeSaufRepere) != 0)
  			ptgen.nom = ((CValeurDynamique) elb).nom();
  		
			// Les points ou droites ayant un nom indispensable doivent voir aussi un nom lors de l'implmentation
			// Ce nom est sauve dans l'lment gnrique
			if ((natureGraphique & (NatureObjet.NToutPoint | NatureObjet.NDroite)) != 0){
				if (listeElements.nomIndispensable((CElementGraphique)elb)) ptgen.nom = ((CElementGraphique) elb).nom;
			}
  		ptgen.estElementFinal = false;
  		if (natureGraphique != 0) ptgen.natureGraphique = natureGraphique;
  		// else ptgen.natureCalcul = natureCalcul;
  		// Attention pas de else car les repres sont  la fois graphiques et non graphiques
  		ptgen.natureCalcul = natureCalcul;
  		add(ptgen);
  	}
  	for (i = nbObjSources; i < nombreTotal; i++)
  	{
  		CElementBase elb = listeElements.get(i);
  		CElementBase elbClone = elb.getClone(listeElements, this);
  		elbClone.estElementFinal = listeElementsFinaux.contains(elb);
  		add(elbClone);
  	}  	
  }
  		
  
  public void setListePourImplementation(CListeObjets listePourImplementationInit) {
  	listePourImplementation = listePourImplementationInit;
  }
  
  /**
   * Cette fonction renvoie l'indice de l'objet dans la liste dans laquelle on implmente le prototye
   */
  @Override
  public int indexOf(Object o) {
  	CElementBase elb = (CElementBase) o;
  	if (elb == null) return -1;
  	if (elb.elementGeneriqueAssocie != null) elb = elb.elementGeneriqueAssocie;
  	int ind = super.indexOf(elb);
  	CListeObjets liste = listePourImplementation;
  	if (liste == null) return ind;
  	if (ind < nbObjSources) {
  		CElementGenerique eltgen = (CElementGenerique) elb;
  		CElementBase eltbase = eltgen.elementAssocie;
  		return liste.indexOf(eltbase);
  	}
  	else return indicePremierObjetPourImplementation + ind - nbObjSources;
  }
    
  /**
   * onction de classe renvoyant true si tous les lments de la liste listeFinaux dpendent
   * bien de au moins un lment de la liste listeSources et false sinon
   * @param listeSources
   * @param listeFinaux
   * @return
   */
  public static boolean verifieObjetsSourcesTousUtilises(CListeObjets listeSources, CListeObjets listeFinaux) {
  	ListIterator<CElementBase> li = listeSources.listIterator();
  	while (li.hasNext()) {
  		CElementBase elb = li.next();
  		if (!listeFinaux.dependDe(elb)) return false;
  	}
  	return true;
  }
  

  /**
   * Fonction renvoyant le nombre d'objets de type calcul parmi les objets sources
   * @return
   */
  public final int nombreSourcesTypeCalcul() {
  	int res = 0;
  	for (int i = 0; i < nbObjSources; i++) {
  		CElementBase elb = get(i);
  		if (elb.estDeNatureCalcul(NatureCalcul.NObjetCalculPourSourcesPrototype)) res++;
  	}
  	return res;
  }
  
  
  /**
   * Fonction renvoyant le nombre d'objets graphiques de type nat
   * @return
   */
  public final int nombreObjetsSourcesGraphiques(long natureGraphique) {
  	int res = 0;
  	for (int i = 0; i < nbObjSources; i++) {
  		CElementBase elb = get(i);
  		if (elb.estDeNature(natureGraphique)) res++;
  	}
  	return res;
  }
 
  /**
   * Fonction renvoyant le nombre d'objets de type calcul de type nat
   * @return
   */
  public final int nombreObjetsSourcesCalcul(long natureCalcul) {
  	int res = 0;
  	for (int i = 0; i < nbObjSources; i++) {
  		CElementBase elb = get(i);
  		if (elb.estDeNatureCalcul(natureCalcul)) res++;
  	}
  	return res;
  }
  
  /**
   * Fonction renvoyant une chane forme de l'aide pour l'implmentation de l'objet ni
   * si elle existe. Cette chane est contenue dans le commentaire du prototype (si elle
   * a t entre) et elle est dlimite en dbut par un caractre # suivi d'une chane
   * reprsentant i suivi d'un caractre : et en fin par un caractre #
   */
  public final String chaineImplementationProto(int i) {
  	StringBuffer stringbuf = new StringBuffer(128);
  	String ch = "#" + Integer.toString(i) + ":";
  	int j = commentaire.indexOf(ch);
  	if (j != -1) {
  		int k = commentaire.indexOf("#", j + ch.length());
  		if (k != -1) stringbuf.append(commentaire.substring(j + ch.length(), k));
  		else stringbuf.append(commentaire.substring(j + ch.length()));
  		int p = stringbuf.indexOf("\n");
  		if (p != -1) stringbuf.delete(p, stringbuf.length());
  		if (stringbuf.length() > 80) stringbuf.delete(80, stringbuf.length());
  	}
  	return stringbuf.toString();
  }
  

 // Chargement de la liste depuis un flux
  public void chargeDepuisFlux (MathGraphDataInputStream inps)  throws IOException {
    numeroVersionPrototype = inps.readCInt();
    if (numeroVersionPrototype < 3) numeroVersion = 7; // Version 4.4
    nom = inps.readCString();
    commentaire = inps.readCString();
    nbObjSources = inps.readCInt();
    nbObjFinaux = inps.readCInt();
    natureSources = inps.readUnsignedByte();
    super.chargePourProtoDepuisFlux(inps);
    // On recherche le objets sources repres et on les met  la fin des objets sources de type calcul
    // On recherche d'abord l'indice du premier objet source graphique qui ne soit pas un repre
    int ind;
    
    boolean trouve = false;
    for (ind = 0; ind < nbObjSources; ind++) {
    	CElementGenerique el = (CElementGenerique) get(ind);
    	if (el.natureGraphique != 0) {
    		trouve = true;
    		break;
    	} else {
		}
    }
    // On regarde si cet objet est suivi de repre. Si c'est le cas on reclasse les repres avant
    // ce premier lment graphique
    if (trouve) {
    	for (int j = ind + 1; j < nbObjSources; j++) {
    		CElementGenerique el = (CElementGenerique) get(j);
    		if (el.natureCalcul == NatureCalcul.NRepere) {
    			remove(j);
    			add(ind, el);
    		}
    	}
    }
     // Il revoir la nature des ojets sources requis
    if (natureSources != ConstructionEltsCalculsSeuls) {
    // On compte les objets sources de type calcul
  	int nb = 0;
  	for (int i = 0; i < nbObjSources; i++) {
  		CElementGenerique gen = (CElementGenerique) get(i);
  		if (gen.natureCalcul != 0) nb++;
  	}
  	if (nb == nbObjSources) natureSources = ConstructionEltsCalculsSeuls;
  	else
  		if (nb == 0) natureSources = ConstructionEltsGraphiquesSeuls;
  		else
  			natureSources = ConstructionEltsMixtes;
  	}
  }
  /* Chargement depuis un flux java */
  public void read(MathGraphDataInputStream inps)  throws IOException {
    numeroVersionPrototype = inps.readInt();
    if (numeroVersionPrototype < 3) numeroVersion = 7; // Version 4.4
    else { // Version 4.7.7 : Le numro de version de prototype passe  4 et le numro de version de liste  11
    	if (numeroVersionPrototype == 3) numeroVersion = 10;
    	else numeroVersion = 11;
    }
    nom = inps.readUTF();
    commentaire = inps.readUTF();
    nbObjSources = inps.readInt();
    nbObjFinaux = inps.readInt();
    natureSources = inps.readUnsignedByte();
    super.readPourProtoDepuisFlux(inps);
  }
  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups)  throws IOException {
  	// Important : quand on enregistre le prototype dans un flux il faut que son membre
  	// listePourImplementation soit null car sinon ce ne sont pas les bons indices des objets
  	// contenus dans le prototype qui vont tre enregistrs.
  	setListePourImplementation(null);
    // oups.writeInt(numeroVersionPrototype); Modifi version 4.7.7
  	oups.writeInt(numeroVersionPrototypes);
    oups.writeUTF(nom);
    oups.writeUTF(commentaire);
    oups.writeInt(nbObjSources);
    oups.writeInt(nbObjFinaux);
    oups.writeByte(natureSources);
    super.writePourProtoDepuisFlux(oups);
  }
}
