/*
 * CSimilitude.java
 *
 * Created on 10 juillet 2004, 12:43
 */

/**
 *
 * @author  Yves
 * @version 
 */
package objets;

import java.awt.geom.Point2D;
import java.io.IOException;
import types.DimensionFenetre;
import types.VecteurDirecteur;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;


public class CSimilitude extends CRotation {
	public CValeurAssocieeAVariable rapport;
  private Point2D.Double point1 = new Point2D.Double();
  // Java : modification par rapport au C++ : on stocke la valeur numrique du rapport
  // pour ne pas avoir  la recalculer plus tard
  double valeurRapport;

  /** Creates new CSimilitude */
  public CSimilitude() {
  }
  
  @Override
	public int natureTransformation() {
		return similitude;
	}  
  
  // Constructeur pour clonage
  public CSimilitude(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CPointAncetre centreInit, CValeurAngle angleInit, CValeurAssocieeAVariable rapportInit) {
    super(liste, impProtoInit, estElementFinalInit, centreInit, angleInit);
    rapport = rapportInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(centre);
    int ind2 = listeSource.indexOf(impProto);        
    CValeurAngle angleClone = (CValeurAngle) angle.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable rapportClone = (CValeurAssocieeAVariable) rapport.getClone(listeSource, listeCible);
    CElementBase ptelb = new CSimilitude(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)), angleClone, rapportClone);
    return ptelb;
  }

  @Override
  void AjouteElementsVisuelsPourHist(CListeObjets liste) {
    liste.add(centre);
  }
  
  String designationTransf() {
    return textes.Chaines.chinfo26;
  }  
  
  String complementPourHist() {
    return super.complementPourHist() + " " + textes.Chaines.chinfo28 + " " + rapport.chaineInfo()
    ;
  }
  
  public boolean dependDe(CElementBase p) {
  	return super.dependDe(p) || rapport.dependDe(p);
  }

  public boolean dependDePourBoucle(CElementBase p) {
  	return super.dependDePourBoucle(p) || rapport.dependDePourBoucle(p);
  }

  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    super.positionne(infoRandom, dimfen);
    rapport.positionne(infoRandom, dimfen);
    existe = existe && rapport.existe();
    if (existe) valeurRapport = rapport.rendValeur(infoRandom);
  }

  public boolean confonduAvec(CElementBase p) {
    CSimilitude pt;
    if (p.getClass() == this.getClass()) {
      pt = (CSimilitude) p;
      return centre.confonduAvec(pt.centre) && angle.confonduAvec(pt.angle)
              && rapport.confonduAvec(pt.rapport);
    } else return false;
  }
  
  public double transformeLongueur(double L) {
  	return Math.abs(valeurRapport)*L;
  }

  public void transformePoint(double x, double y, Point2D.Double pointr) {
    // On cre d'abord l'image par l'objet rotation dont CSimilitude
    // hrite puis on prend l'image par une homothtie de rapport k
    super.transformePoint(x,y,point1);
    pointr.x = centre.x + valeurRapport*(point1.x - centre.x);
    pointr.y = centre.y + valeurRapport*(point1.y - centre.y);
  }

  public void transformeVecteur(VecteurDirecteur u, VecteurDirecteur ur) {
    VecteurDirecteur v = new VecteurDirecteur();
    super.transformeVecteur(u, v);
    ur.x = valeurRapport* v.x;
    ur.y = valeurRapport* v.y;
  }
  
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    rapport = new CValeurAssocieeAVariable();
    rapport.chargeDepuisFlux(inps, list);
  } 
  
  /* Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    rapport = new CValeurAssocieeAVariable();
    rapport.read(inps, list);
  }
  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    rapport.write(oups, list);
  }     
}
