/*
 * CSuiteRec.java
 *
 * Created on 20 juillet 2004, 12:12
 */

/**
 *
 * @author  Yves
 * @version
 */
package objets;

import java.io.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CSuiteRec extends CCalculAncetre {
  public CFonc fonctionAssociee;
  public CValeurAssocieeAVariable nombreTermes;
  int nbt;
  public CValeurAssocieeAVariable premierTerme;
  int indiceDernierTermeExistant;
  public double[] valeurs; // Pointeur sur un tableau contenant les valeurs
  // des termes successifs
  
  /** Creates new CSuiteRec */
  public CSuiteRec() {
  }
  
  /**
   * Constructeur pour cration d'objets
   * @param liste
   * @param nombreTermesInit
   * @param premierTermeInit
   */
  public CSuiteRec(CListeObjets liste, CValeurAssocieeAVariable nombreTermesInit, CValeurAssocieeAVariable premierTermeInit) {
  	super(liste, null, false, "");
  	fonctionAssociee = null;
    nombreTermes = nombreTermesInit;
    premierTerme = premierTermeInit;
    metAJourNombreTermes(); 	
  }
  /** Constructeur pour clonage
   * 
   * @param liste
   * @param impProtoInit
   * @param estElementFinalInit
   * @param nomCalculInit
   * @param fonctionAssocieeInit
   * @param nombreTermesInit
   * @param premierTermeInit
   */
  public CSuiteRec(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    String nomCalculInit, CFonc fonctionAssocieeInit,
    CValeurAssocieeAVariable nombreTermesInit, CValeurAssocieeAVariable premierTermeInit) {
    super(liste, impProtoInit, estElementFinalInit, nomCalculInit);
    fonctionAssociee = fonctionAssocieeInit;
    nombreTermes = nombreTermesInit;
    premierTerme = premierTermeInit;
    metAJourNombreTermes();
  }
  
  static int NombreMaxiTermesSuiteRec = 2000;
  
  void metAJourNombreTermes() {
    double val = nombreTermes.rendValeur(false);
    if ((val == Math.floor(val)) && (val >= 1) && (val < NombreMaxiTermesSuiteRec)) nbt = (int) val;
    else nbt = 2; // Pas d'importance car dans ce cas la suite n'existe pas
    valeurs = new double[nbt];
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(fonctionAssociee);
    int ind2 = listeSource.indexOf(impProto);
    CValeurAssocieeAVariable premierTermeClone =
      (CValeurAssocieeAVariable) premierTerme.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable nombreTermesClone =
      (CValeurAssocieeAVariable) nombreTermes.getClone(listeSource, listeCible);
    CElementBase ptelb = new CSuiteRec(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CFonc)listeCible.get(ind1, CFonc.class), nombreTermesClone,
      premierTermeClone);
    return ptelb;
  }
  
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    indiceDernierTermeExistant = ((CSuiteRec) ptel).indiceDernierTermeExistant;
    for (int i = 0; i <= indiceDernierTermeExistant; i++) {
      valeurs[i] = ((CSuiteRec) ptel).valeurs[i];
    }
  }
  
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || premierTerme.dependDe(p) || fonctionAssociee.dependDe(p) || nombreTermes.dependDe(p);
  }
  
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || premierTerme.dependDePourBoucle(p) || fonctionAssociee.dependDePourBoucle(p)
    || nombreTermes.dependDePourBoucle(p);
  }
  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double u,v;
    nombreTermes.positionne(infoRandom, dimfen);
    premierTerme.positionne(infoRandom, dimfen);
    existe = fonctionAssociee.existe && premierTerme.existe() && nombreTermes.existe();
    if (!existe) return;
    double dnbt = Math.floor(nombreTermes.rendValeur(false) + 0.5);
    existe = (dnbt >= 1) && (dnbt <= NombreMaxiTermesSuiteRec);
    if (!existe) return;
    if ((int) dnbt != nbt) {
      nbt = (int) dnbt;
      metAJourNombreTermes();
    }
    
    u = premierTerme.rendValeur(infoRandom);
    valeurs[0] = u;
    indiceDernierTermeExistant = 0;
    int i = 1;
    while (i < nbt) {
      try {
        v = fonctionAssociee.calcul.resultatFonction(infoRandom, u);
        if (Double.isNaN(v) || Double.isInfinite(v)) {
          break;
        }
        u = v;
        valeurs[i] = u;
        i++;
      } catch (ErreurCalculException e) {
        break;
      }
    }
    i--;
    indiceDernierTermeExistant = i;
  }
  
  @Override
	public boolean confonduAvec(CElementBase p) {
    CSuiteRec pt;
    if (p.getClass() == this.getClass()) {
      pt = (CSuiteRec) p;
      return (fonctionAssociee == pt.fonctionAssociee) && (premierTerme.confonduAvec(pt.premierTerme)
      		&& (nombreTermes.confonduAvec(pt.nombreTermes)));
    }
    else return false;
	}
  
  public double rendValeurFonction(boolean infoRandom, double parametre) throws ErreurCalculException {
    if (!existe || (parametre < 0) || (parametre != Math.floor(parametre))
    || (parametre > indiceDernierTermeExistant)) {
      throw new ErreurCalculException();
    } else return valeurs[(int) parametre];
  }
  

  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NSuiteRecurrenteReelle;
  }
  
  /**
   * Pour savoir si le calcul est une fonction relle, une suite relle ou non
   * Redfini seulement pour CFonc, CFoncComplexe, CSuiteRec et CSuiteRecComplexe;
   */ 
  public boolean estFonctionOuSuite() {
    return true;
  }
  
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps,list);
    nombreTermes = new CValeurAssocieeAVariable();
    if (nVersion == 1) {
      int nbtermes = inps.readCInt();
      nombreTermes.donneValeur(nbtermes);
    }
    else nombreTermes.chargeDepuisFlux(inps, list);
    premierTerme = new CValeurAssocieeAVariable();
    premierTerme.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    fonctionAssociee = (CFonc) (list.get(ind1,CFonc.class));
    // metAJourNombreTermes();
    // Le nombre de valeurs tant dynamique, on ne peut pas le connatre au chargement
    // On met 1 terme arbitrairement et cela sera rectifi dans positionne();
    nbt = 1;
    valeurs = new double[nbt];
    nVersion = numeroVersion(); // On remet tout  zro pour la version java
  }
  /* Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    nombreTermes = new CValeurAssocieeAVariable();
    nombreTermes.read(inps, list);
    premierTerme = new CValeurAssocieeAVariable();
    premierTerme.read(inps, list);
    int ind1 = inps.readInt();
    fonctionAssociee = (CFonc) (list.get(ind1,CFonc.class));
    // metAJourNombreTermes();
    // Le nombre de valeurs tant dynamique, on ne peut pas le connatre au chargement
    // On met 1 terme arbitrairement et cela sera rectifi dans positionne();
    nbt = 1;
    valeurs = new double[nbt];    
  }
  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    nombreTermes.write(oups, list);
    premierTerme.write(oups, list);
    int ind1 = list.indexOf(fonctionAssociee);
    oups.writeInt(ind1);
  }    
}
