/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import types.Complexe;
import types.DimensionFenetre;
import types.ErreurCalculException;
import types.NatureCalcul;

// Suite rcurrente complexe de la forme u(n+1)=f[n,u(n)] o f est une fonction complexe de deux variables
public class CSuiteRecComplexe2 extends CSuiteRecComplexe {
  
  public CSuiteRecComplexe2() {
  }
  
  /**
   * Constructeur pour cration d'objets
   * @param liste
   * @param nombreTermesInit
   * @param premierTermeInit
   */
  public CSuiteRecComplexe2(CListeObjets liste, CValeurAssocieeAVariable nombreTermesInit, CValeurAssocieeComp premierTermeInit) {
  	super(liste, nombreTermesInit, premierTermeInit);
  }
  
  // Constructeur pour clonage
  public CSuiteRecComplexe2(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    String nomCalculInit, CFoncComplexeNVar fonctionAssocieeInit,
    CValeurAssocieeAVariable nombreTermesInit, CValeurAssocieeComp premierTermeInit) {
    super(liste, impProtoInit, estElementFinalInit, nomCalculInit, fonctionAssocieeInit, nombreTermesInit, premierTermeInit);
  }
    
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(fonctionAssociee);
    int ind2 = listeSource.indexOf(impProto);
    CValeurAssocieeComp premierTermeClone =
      (CValeurAssocieeComp) premierTerme.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable nombreTermesClone =
      (CValeurAssocieeAVariable) nombreTermes.getClone(listeSource, listeCible);
    CElementBase ptelb = new CSuiteRecComplexe2(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CFoncComplexeNVar)(listeCible.get(ind1, CFoncComplexeNVar.class)), nombreTermesClone, premierTermeClone);
    return ptelb;
  }
    
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    premierTerme.positionne(infoRandom, dimfen);
    nombreTermes.positionne(infoRandom, dimfen);
    existe = fonctionAssociee.existe && premierTerme.existe() && nombreTermes.existe();
    if (!existe) return;
    double dnbt = Math.floor(nombreTermes.rendValeur(false) + 0.5);
    existe = (dnbt >= 1) && (dnbt <= NombreMaxiTermesSuiteRec);
    if (!existe) return;
    if ((int) dnbt != nbt) {
      nbt = (int) dnbt;
      metAJourNombreTermes();
    }    
    // Modifi version 3.1.2 pour autoriser la remise  jour des valeurs renvoyes par rand
    // premierTerme.rendValeurComplexe(infoRandom, u);
    premierTerme.rendValeurComplexe(true, u);
    valeurs[0].set(u);
    indiceDernierTermeExistant = 0;
    int i = 1;
    while (i < nbt) {
      try {
        listeProprietaire.initialiseNombreIterations();
        // fonctionAssociee.calcul.resultatFonctionComplexe(infoRandom, u, v);
        Complexe[] arg = new Complexe[2];
        arg[0] = new Complexe();
        arg[0].x = i - 1;
        arg[0].y = 0;
        arg[1] = new Complexe(u);
        
        fonctionAssociee.calcul.resultatFonctionComplexe(true, arg, v);
        if (v.isNanOrInfinite())  break;
        u.set(v);
        valeurs[i].set(u);
        i++;
      } catch (ErreurCalculException e) {
        break;
      }
    }
    i--;
    indiceDernierTermeExistant = i;
  }
  
  @Override
	public boolean confonduAvec(CElementBase p) {
    CSuiteRecComplexe2 pt;
    if (p.getClass() == this.getClass()) {
      pt = (CSuiteRecComplexe2) p;
      return (fonctionAssociee == pt.fonctionAssociee) && (premierTerme.confonduAvec(pt.premierTerme)
      		&& (nombreTermes.confonduAvec(pt.nombreTermes)));
    }
    else return false;
	}
 
  @Override
  public String infoPourHistorique() {
  	String ch = getString("chinfo184") + "\n" + getString("chinfo84") + " " + fonctionAssociee.nom() + "\n" +
  	getString("chinfo85") + " " + premierTerme.chaineInfo() + "\n" +
  	getString("chinfo86") + " " + nombreTermes.chaineInfo();
    if (!existe) ch = ch + "\n" + getString("ch18");
    return ch;
  }
    
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NSuiteRecurrenteComplexe2;
  }
  
}
