/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package objets;

import java.awt.Color;
import java.awt.geom.Point2D;
import java.io.IOException;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import types.Couleur;
import types.DimensionFenetre;
import types.MotifPoint;
import types.StyleFleche;
import types.StyleTrait;
import types.VecteurDirecteur;

public class CTranslationParCoord extends CTransformation {
	public CRepere rep;
	public CValeurAssocieeAVariable x; // Premire coordonne du vecteur de la translation
	public CValeurAssocieeAVariable y; // Deuxime coordonne du vecteur de la translation
  private VecteurDirecteur valeurVecteur;
	
  /**
   * Default Constructor
   */
	public CTranslationParCoord() {
		x = new CValeurAssocieeAVariable();
		y = new CValeurAssocieeAVariable();
		valeurVecteur = new VecteurDirecteur();
	}
	
  @Override
	public int natureTransformation() {
		return translationParCoord;
	}
  /** Constructeur pour clonage */
  public CTranslationParCoord(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
  	CRepere repInit, CValeurAssocieeAVariable xInit, CValeurAssocieeAVariable yInit) {
    super(liste, impProtoInit, estElementFinalInit);
    valeurVecteur = new VecteurDirecteur();
    rep = repInit;
    x = xInit;
    y = yInit;
  }
	
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(rep);
    int ind2 = listeSource.indexOf(impProto);        
    CValeurAssocieeAVariable xClone = (CValeurAssocieeAVariable) x.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable yClone = (CValeurAssocieeAVariable) y.getClone(listeSource, listeCible);
    CElementBase ptelb = new CTranslationParCoord(listeCible, (CImplementationProto)listeCible.get(ind2, CImplementationProto.class),
      estElementFinal,(CRepere)(listeCible.get(ind1, CRepere.class)), xClone, yClone);
    return ptelb;    
  }
	
  @Override
  void AjouteElementsVisuelsPourHist(CListeObjets liste) {
    CPointImage ptim = new CPointImage(liste, null, false, Color.black,
	       MotifPoint.petitRond, 2, rep.O, this);
    ptim.cache();
    liste.add(ptim);
    CVecteur vec = new CVecteur(liste, null, false,
      Couleur.TraduitCouleur(2), false, StyleTrait.traitEpais,
      rep.O, ptim, StyleFleche.FlecheCourtePleine);
    liste.add(vec);
  }
  
  @Override
  String designationTransf() {
    return getString("chinfo19");
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || rep.dependDe(p) || x.dependDe(p) || y.dependDe(p));
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || rep.dependDePourBoucle(p) ||  x.dependDePourBoucle(p) || y.dependDePourBoucle(p));
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	x.positionne(infoRandom, dimfen);
  	y.positionne(infoRandom, dimfen);
    existe = rep.existe && x.existe() && y.existe();
    if (existe) {
    	valeurVecteur.x = x.rendValeur(infoRandom)/rep.unitex;
    	valeurVecteur.y = y.rendValeur(infoRandom)/rep.unitey;
    }
  }
  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CTranslationParCoord pt;
    if (p.getClass() == this.getClass()) {
      pt = (CTranslationParCoord) p;
      return (rep == pt.rep) && x.confonduAvec(pt.x) && y .confonduAvec(pt.y);
    } else return false;
  }
  
  @Override
  public double transformeLongueur(double L) {
    return L;
  }

  @Override
  public void transformePoint(double x, double y, Point2D.Double pointr) {
    pointr.x = x + valeurVecteur.x*rep.u.x + valeurVecteur.y*rep.v.x;
    pointr.y = y + valeurVecteur.x*rep.u.y + valeurVecteur.y*rep.v.y;;
  }

  @Override
  public void transformeVecteur(VecteurDirecteur u, VecteurDirecteur ur) {
    u.setCopy(ur);
  }
  
  /**
   * Fonction renvoyant true lorsque les images par ce type de transformation peuvent tre modifies
   * via un menu. C'est le cas pour les rotations, homothties, similitudes, inversions pour lequels
   * il faudra redfinir  cette fonction
   * @return
   */
  @Override
  public boolean imageModifiableParMenu() {
  	return true;
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    rep = (CRepere) (list.get(ind1,CRepere.class));
    x.read(inps, list);
    y.read(inps, list);    
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(rep);
    oups.writeInt(ind1);
    x.write(oups, list);
    y.write(oups, list);
  }  

}
