/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;
import Applet.AppletMathGraph32;
import java.awt.Color;
import objets.*;
import types.Couleur;
import types.MotifPoint;
import types.StyleEncadrement;
import types.StyleFleche;
import types.StyleMarqueAngle;
import types.StyleTrait;
import types.VecteurDirecteur;

/**
 *
 * @author Yves
 */
public class OutilMesAngOr extends OutilPar3pts {
  
  /** Creates a new instance of OutilSeg */
  public OutilMesAngOr(AppletMathGraph32 appletMereInit) {
    super(appletMereInit, AppletMathGraph32.iconMesureAngleOriente);
  }

  /**
   * Les outils descendant de OutilPar2Pts et necessitant que les points dsigns
   * soient nomms s'ils n'ont pas de noms devront redfinir cette fonction en renvoyant true
   * Cela servira pour l'outil de mesure de longueur
   * @return true si les points dsigns doivent tre nomms
   */
  boolean nomsPointsIndisp() {
    return true;
  }
  
  public int idMenu() {
    return 32814;
  }
  
  public String toolTip() {
    return getString("toolTip30");
  }
  
  /* Appel pour crer finalement l'objet une fois que tous les objets dont il
   * dpend ont t dsigns */
  public void creeObjet() {
    appletMere.videDesignation();
    CMesureAngleOriente mes = new CMesureAngleOriente
            (appletMere.listePr, null, false , point1, point2, point3);
    if (appletMere.ajouteElementAvecVerification(mes)) {
      objetCree = true; // Necessaire  cause du fait que point1 et point2 ont t ventuellement
      appletMere.gestionnaire.enregistreFigureEnCours(getString("chaineOutil26"));
      appletMere.dialogueAvertissement(getString("ch29") + " " + mes.nom() + " " +  getString("ch28"));
    }
    select();
  }

  /** Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de l'applet
   * au fur et  mesure que l'on dsigne les obejts  la souris
   * Doit tre redfini pour chaque outil utilisant une aide visuelle  la cration
   */
  @Override
  public void ajouteObjetsVisuels() {    
    CListeObjets list = appletMere.listeObjetsVisuels;
    // On cre deux segments en pointills
  	CSegment pts1 = new CSegment(list, null, false, point1, point2);
  	appletMere.ajouteObjetVisuel(pts1);
  	CSegment pts2 = new CSegment(list, null, false, point2, appletMere.pointSouris);
  	appletMere.ajouteObjetVisuel(pts2);
    // On cre une marque d'angle orient
    CMarqueAngleOriente mar = new CMarqueAngleOriente(list, null, false,
            Color.BLACK, false, StyleTrait.traitEpais, StyleMarqueAngle.marqueSimple,
            12, StyleFleche.FlecheCourtePleine, point1, point2, appletMere.pointSouris);
    appletMere.ajouteObjetVisuel(mar);
    
    // On cre un point par abscisse sur le segment joignant point2 et point1 
    // une distance de 20 pixels de point2
    VecteurDirecteur vec = new VecteurDirecteur(point2, point1);
    double d = 20/vec.norme();
    CValeurAssocieeAVariable ab = new CValeurAssocieeAVariable(list, d);
    CPointParAbscisse pt1 = new CPointParAbscisse(list, null, false, Color.BLACK, 
            false, 0d, 0d, true, "", 2, MotifPoint.petitRond, point2, point1, ab, false);
    appletMere.ajouteObjetVisuel(pt1);
    // On cre un petit arc de cercle de centre point2, commenant par point 1
    // et dirig vers le pointeur souris
    CArcDeCercle arc = new CArcDeCercle(list, null, false, Color.BLACK,
            true, StyleTrait.traitFin, point2, pt1, appletMere.pointSouris, null);
    appletMere.ajouteObjetVisuel(arc);
    // On cre une bissectrice
    CBissectrice bis = new CBissectrice(list, null, false, Color.BLACK, StyleTrait.traitFin,
            point1, point2, appletMere.pointSouris, 2);
    bis.cache();
    appletMere.ajouteObjetVisuel(bis);
    // On cre l'intersection de l'arc et de la bissectrice
    CIntDroiteCercle inter = new CIntDroiteCercle(list, null, false, bis, arc);
    appletMere.ajouteObjetVisuel(inter);
    CPointLieBipoint ptl1 = new CPointLieBipoint(list, null, false, Color.BLACK,
            false, 0d, 0d, true, "", 2, MotifPoint.petitRond, inter, 1, false);
    appletMere.ajouteObjetVisuel(ptl1);
    CPointLieBipoint ptl2 = new CPointLieBipoint(list, null, false, Color.BLACK,
            false, 0d, 0d, true, "", 2, MotifPoint.petitRond, inter, 2, false);
    appletMere.ajouteObjetVisuel(ptl2);
    // Un des deux points d'intersection n'existera pas mais on ne sait pas lequel
    // On cre un commentaire contenant un point d'intersection li  chacun d'eux
    CCommentaire com1 = new CCommentaire(list, null, false, Color.BLACK, 0d, 0d,
                -4d, -6d, false, ptl1, 1, StyleEncadrement.Sans, false, Couleur.TraduitCouleur(0), "?",
                CValeurAffichee.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalCentre);
    appletMere.ajouteObjetVisuel(com1);
    CCommentaire com2 = new CCommentaire(list, null, false, Color.BLACK, 0d, 0d,
                -4d, -6d, false, ptl2, 1, StyleEncadrement.Sans, false, Couleur.TraduitCouleur(0), "?",
                CValeurAffichee.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalCentre);
    appletMere.ajouteObjetVisuel(com2);
  }
/**
   * Fonction renvoyant la chane  afficher  l'tape i (i >= 1) comme indication
   * Doit tre redfinie par les descendants devant crire plusieurs indications
   * @param i : l'indice
   * @return
   */          
  public String chaineIndication (int indEtape) {
    switch(indEtape) {
      case 1 : return getString("mesAngOr1");
      case 2 : return getString("mesAngOr2");
      case 3 : return getString("mesAngOr3");
      
      default: return "";
    }
  }
  
}
