/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.*;

import dialogs.*;
import objets.*;
import types.*;
import Applet.*;

/**
 *
 * @author Yves
 */
public class OutilRot extends Outil {
  CPointAncetre point;
  CElementGraphique objet;
  private int nombreImagesCrees;
  CRotation ptRot;
  
  /** Creates a new instance of OutilRot */
  public OutilRot(AppletMathGraph32 appletMereInit) {
    super(appletMereInit, true, AppletMathGraph32.iconCreerImageParRotation);
  }
  
  public int idMenu() {
    return 32803;
  }  

  public String toolTip() {
    return getString("toolTip18");
  }
  
  /** Appel lorsque l'outil est slectionn */
  public void select() {
    super.select();
    point = null;
    objet = null;
    ptRot = null;
    nombreImagesCrees = 0;
    designationDe(NatureObjet.NToutPoint);
    appletMere.indication(getString("rotation1"));
  }
  
  /**
   * Il est ncessaire de regarder quand l'outil est dselectionn si on a vraiment cr des images
   * par une transformation. Sinon il faut supprimer le dernier objet qui est une homothtie
   */
  @Override
  public void deselect() {
    CListeObjets liste = appletMere.listePr;
    // Si on a utilis une similitude nouvelle et rien n'a t cr c'est le dernier lment
    if (ptRot == liste.get(liste.size() - 1)){
    	appletMere.detruitDernierElement();
    }
    super.deselect();
  }
  

  /* Appel pour crer finalement l'objet une fois que tous les objets dont il
   * dpend ont t dsigns */
  public void creeObjet() {
    // CListeObjets liste = appletMere.listePr;
  	CListeObjets liste = appletMere.listeConstruction;  // Modifi version 4.6
    if (nombreImagesCrees == 0) {
    	CTransformation rot = liste.transformationConfondueAvec(ptRot);
    	if (rot != null) {
    		ptRot = (CRotation) rot;
     	}
    	else appletMere.ajouteElement(ptRot);
    }
    Int nbImagesCrees = new Int();
    CElementGraphique elg = imageParTransformation(liste,  objet, ptRot, nbImagesCrees);
    if (appletMere.ajouteElementsImageAvecVerification(elg, nbImagesCrees.getValue(), ptRot)) {
    	appletMere.gestionnaire.enregistreFigureEnCours(getString("chaineOutil14"));
    	nombreImagesCrees++;
  		appletMere.repaintFigureAvecCalcul();    	
    }
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    CListeObjets liste = appletMere.listePr;
    if (point == null) {
      appletMere.indication(getString("rotation2"));
      point = (CPointAncetre) elg;
      excluDeDesignation(elg);
      ajouteClignotementDe(elg);
      resetClignotement();
      CValeurAngle ang = new CValeurAngle(liste, 0, "0");
      ptRot = new CRotation(liste, null, false, point, ang);
      // On cre un dialogue modal permettant d'entrer l'angle de la rotation
      AngleRotDlg angleRotDlg = new AngleRotDlg(appletMere, ptRot);
      angleRotDlg.setLocationRelativeTo(appletMere); // Pour centrer la boite de dialogue sur l'cran
      // On redessine la figure pour liminer la dsignation ce point affiche pour le centre
      appletMere.videDesignation();
      appletMere.paneFigure.repaint();
      angleRotDlg.setVisible(true);
      // On regarde si la bote de dialogue a bien t valide par OK
      if (angleRotDlg.retourOK()) {
        ajouteObjetsVisuels();
        designationDe(appletMere.natureObjetsPourImages());
        resetClignotement();
        appletMere.indication(getString("rotation3"));

      }
      else {
        annuleClignotement();
        select(); 
      }      
    }
    else {
      objet = elg;
      creeObjet();
    }
  }
  
  /* Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de l'applet
   * au fur et  mesure que l'on dsigne les obejts  la souris
   * Doit tre redfini pour chaque outil utilisant une aide visuelle  la cration
   */
  public void ajouteObjetsVisuels() {
    CListeObjets liste = appletMere.listeObjetsVisuels;
  	// On cre un segment en pointills joignant le centre au point souris
  	CSegment pts1 = new CSegment(liste, null, false, point, appletMere.pointSouris);
    appletMere.ajouteObjetVisuel(pts1);
    appletMere.ajouteObjetVisuel(ptRot);
    CPointImage ptim = new CPointImage(liste, null, false, Couleur.TraduitCouleur(1),
      MotifPoint.petitRond, 2, appletMere.pointSouris, ptRot);
    appletMere.ajouteObjetVisuel(ptim);
    // On cre un deuxime segment joigant le centre au point image
  	CSegment pts2 = new CSegment(liste, null, false, point, ptim);
    appletMere.ajouteObjetVisuel(pts2);
    // On cre une marque d'angle orient pour visualiser la rotation
    CMarqueAngleOriente ptmar = new CMarqueAngleOriente(liste, null, false,
      Couleur.TraduitCouleur(1), false, StyleTrait.traitFin, StyleMarqueAngle.marqueSimple, 18, StyleFleche.FlecheCourtePleine, 
      appletMere.pointSouris, point, ptim);
    appletMere.ajouteObjetVisuel(ptmar);
  }
}
