/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.*;

import objets.*;
import types.*;
import Applet.*;

/**
 *
 * @author Yves
 */
public class OutilSymCentrale extends Outil {
  private CPointAncetre point;
  private CElementGraphique objet;
  private int nombreImagesCrees;
  private CSymetrieCentrale ptSym;
  
  /** Creates a new instance of OutilTranslation */
  public OutilSymCentrale(AppletMathGraph32 appletMereInit) {
    super(appletMereInit, true, AppletMathGraph32.iconCreerImageParSymetrieCentrale);
  }

  public int idMenu() {
    return 32802;
  }
  
  public String toolTip() {
    return getString("toolTip22");
  }

  /** Appel lorsque l'outil est slectionn */
  public void select() {
    super.select();
    point = null;
    objet = null;
    nombreImagesCrees = 0;
    designationDe(NatureObjet.NToutPoint);
    appletMere.indication(getString("symCentrale1"));
  }
  
  /**
   * Il est ncessaire de regarder quand l'outil est dselectionn si on a vraiment cr des images
   * par une transformation. Sinon il faut supprimer le dernier objet qui est une homothtie
   */
  @Override
  public void deselect() {
    CListeObjets liste = appletMere.listePr;
    if (ptSym == liste.get(liste.size() - 1)){
    	appletMere.detruitDernierElement();
    }
    super.deselect();
  }
  
  
  @Override
  public void creeObjet() {
    // CListeObjets liste = appletMere.listePr;
  	CListeObjets liste = appletMere.listeConstruction;  // Modifi version 4.6
    if (nombreImagesCrees == 0) {
    	ptSym = new CSymetrieCentrale(liste, null, false, point);
    	CTransformation sym = liste.transformationConfondueAvec(ptSym);
    	if (sym != null) {
    		ptSym = (CSymetrieCentrale) sym;
     	}
    	else appletMere.ajouteElement(ptSym);
    }
    Int nbImagesCrees = new Int();
    CElementGraphique elg = imageParTransformation(liste,  objet, ptSym, nbImagesCrees);
    if (appletMere.ajouteElementsImageAvecVerification(elg, nbImagesCrees.getValue(), ptSym)) {
    	nombreImagesCrees++;
    	if (nombreImagesCrees != 0) {
    		appletMere.gestionnaire.enregistreFigureEnCours(getString("chaineOutil18"));
    	}
    	appletMere.repaintFigureAvecCalcul();    	
    }
  }
  
  /** Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de l'applet
   * au fur et  mesure que l'on dsigne les objets  la souris
   * Doit tre redfini pour chaque outil utilisant une aide visuelle  la cration
   */
  public void ajouteObjetsVisuels() {
    CListeObjets liste = appletMere.listeObjetsVisuels;
    CSymetrieCentrale sym = new CSymetrieCentrale(liste, null, false, point);
    appletMere.ajouteObjetVisuel(sym);
    CPointImage ptim = new CPointImage(liste, null, false, Couleur.TraduitCouleur(1),
      MotifPoint.petitRond, 2, appletMere.pointSouris, sym);
    appletMere.ajouteObjetVisuel(ptim);
    CMilieu ptm = new CMilieu(liste, null, false, Couleur.TraduitCouleur(1),
       MotifPoint.petitRond, appletMere.pointSouris, ptim);
    appletMere.ajouteObjetVisuel(ptm);
    CSegment seg1 = new CSegment(liste, null, false, Couleur.TraduitCouleur(1),
       false, StyleTrait.traitPointille, appletMere.pointSouris, ptm);
    appletMere.ajouteObjetVisuel(seg1);
    CSegment seg2 = new CSegment(liste, null, false, Couleur.TraduitCouleur(1),
       false, StyleTrait.traitPointille, ptm, ptim);
    appletMere.ajouteObjetVisuel(seg2);
    CMarqueSegment mar1 = new CMarqueSegment(liste, null, false, Couleur.TraduitCouleur(1),
        false, StyleTrait.traitEpais, MotifMarqueSegment.MarqueSimple, seg1);
    appletMere.ajouteObjetVisuel(mar1);
    CMarqueSegment mar2 = new CMarqueSegment(liste, null, false, Couleur.TraduitCouleur(1),
       false, StyleTrait.traitEpais,
      MotifMarqueSegment.MarqueSimple, seg2);
    appletMere.ajouteObjetVisuel(mar2);
  }

  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (point == null) {
      appletMere.indication(getString("symAxiale2"));
      point = (CPointAncetre) elg;
      excluDeDesignation(elg);
      ajouteClignotementDe(elg);
      ajouteObjetsVisuels();
      designationDe(appletMere.natureObjetsPourImages());
      resetClignotement();
    }
    else {
	    objet = elg;
	    creeObjet();
    }
  } 
  
}
