/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;
import java.awt.*;
import objets.*;
import types.*;
import Applet.*;

/**
 *
 * @author Yves
 */
public class OutilTranslation extends Outil {
  private CPointAncetre point1;
  private CPointAncetre point2;
  private CVecteur vecteur;
  private CElementGraphique objet;
  private int nombreImagesCrees;
  CTransformation transfor;
  
  /** Creates a new instance of OutilTranslation */
  public OutilTranslation(AppletMathGraph32 appletMereInit) {
    super(appletMereInit, true, AppletMathGraph32.iconCreerImageParTranslation);
  }

  public int idMenu() {
    return 32845;
  }
    
  public String toolTip() {
    return getString("toolTip23");
  }
  
  /** Appel lorsque l'outil est slectionn */
  public void select() {
    super.select();
    point1 = null;
    point2 = null;
    vecteur = null;
    objet = null;
    nombreImagesCrees = 0;
    designationDe(NatureObjet.NToutPoint | NatureObjet.NVecteur);
    appletMere.outilPointageActif = appletMere.outilPointageCre;
    appletMere.indication(getString("translation1"));
  }
  
  /** On regarde si on n'a pas cr d'image. Dans ce cas il faut dtruire la translation
   * dans le cas o on a cliqu sur deux points
   */
  @Override
  public void deselect() {
    CListeObjets liste = appletMere.listePr;
    if (transfor == liste.get(liste.size() - 1)){
    	appletMere.detruitDernierElement();
    }
  	super.deselect();
  }
  
  
  // Modifi version 4.6 pour n'utiliser que les objets construits
  @Override
  public void creeObjet() {
    // CListeObjets liste = appletMere.listePr;
  	CListeObjets liste = appletMere.listeConstruction;  // Modifi version 4.6
    if (nombreImagesCrees == 0) {
    	if (vecteur == null) {
    		CTranslation translation;
	    	translation = new CTranslation(liste, null, false, point1, point2);
	    	CTransformation trans = liste.transformationConfondueAvec(translation);
	    	if (trans != null) {
	    		translation = (CTranslation) trans;
	     	}
	    	else appletMere.ajouteElement(translation);
	    	transfor = translation;
    	}
    	else {
    		CTransParVect transVect;
	    	transVect = new CTransParVect(liste, null, false, vecteur);
	    	CTransformation trans = liste.transformationConfondueAvec(transVect);
	    	if (trans != null) {
	    		transVect = (CTransParVect) trans;
	     	}
	    	else appletMere.ajouteElement(transVect); 
	    	transfor = transVect;
    	}
    }
    Int nbImagesCrees = new Int();
    CElementGraphique elg = imageParTransformation(liste,  objet, transfor, nbImagesCrees);
    if (appletMere.ajouteElementsImageAvecVerification(elg, nbImagesCrees.getValue(), transfor)) {
    	nombreImagesCrees++;
    	if (nombreImagesCrees != 0) {
    		appletMere.gestionnaire.enregistreFigureEnCours(getString("chaineOutil19"));
    	}
  		appletMere.repaintFigureAvecCalcul();    	
    }
  }
  
  /** Mthode ajoutant les objets visuels  la liste listeObjetsVisuels de l'applet
   * au fur et  mesure que l'on dsigne les obejts  la souris
   * Doit tre redfini pour chaque outil utilisant une aide visuelle  la cration
   */
  @Override
  public void ajouteObjetsVisuels() {
    CListeObjets liste = appletMere.listeObjetsVisuels;
    if (vecteur == null) {
	    CTranslation trans = new CTranslation(liste, null, false, point1, point2);
	    appletMere.ajouteObjetVisuel(trans);
	    CPointImage ptim = new CPointImage(liste, null, false, Couleur.TraduitCouleur(1),
	       MotifPoint.petitRond, 2, appletMere.pointSouris, trans);
	    appletMere.ajouteObjetVisuel(ptim);
	    CVecteur vec = new CVecteur(liste, null, false,
	      Couleur.TraduitCouleur(1), false, StyleTrait.traitPointille,
	      appletMere.pointSouris, ptim, StyleFleche.FlecheCourtePleine);
	    appletMere.ajouteObjetVisuel(vec);
    }
    else {
    	CTransParVect trans = new CTransParVect(liste, null, false, vecteur);
	    appletMere.ajouteObjetVisuel(trans);
	    CPointImage ptim = new CPointImage(liste, null, false, Couleur.TraduitCouleur(1),
	       MotifPoint.petitRond, 2, appletMere.pointSouris, trans);
	    appletMere.ajouteObjetVisuel(ptim);
	    CVecteur vec = new CVecteur(appletMere.listeObjetsVisuels, null, false,
	      Couleur.TraduitCouleur(1), false, StyleTrait.traitPointille,
	      appletMere.pointSouris, ptim, StyleFleche.FlecheCourtePleine);
	    appletMere.ajouteObjetVisuel(vec);
    }
  }

  /* Appel lorsqu'un objet a t cliqu pour tre trait par l'outil
   * par exemple le premier point pour crer une droite passant par deux points */
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
  	if ((vecteur != null) || ((point1 != null) && (point2 != null))) {
  		// Possibilit d'image multiples
    	// Cration d'images multiples possibles
      objet = elg;
      creeObjet();  		
  	}
  	else {
	  	if (elg.getNature() == NatureObjet.NVecteur) {
	  		if (vecteur == null) {
		      appletMere.indication(getString("translation3"));
		      excluDeDesignation(elg);
		  		vecteur = (CVecteur) elg;
	        ajouteObjetsVisuels();
	        ajouteClignotementDe(elg);
	        designationDe(appletMere.natureObjetsPourImages());
	      }
	  	}
	  	else {
		    if (point1 == null) {
		      appletMere.indication(getString("translation2"));
		      point1 = (CPointAncetre) elg;
		      ajouteClignotementDe(elg);
		      designationDe(NatureObjet.NToutPoint);
		      resetClignotement();
		    }
		    else {
		      if (point2 == null) {
		        appletMere.indication(getString("translation3"));
		        point2 = (CPointAncetre) elg;
		        // Le vecteur servant aux clignotements
		        CVecteur vecteurClignotant = new CVecteur(appletMere.listePr, null, false,
			          Couleur.TraduitCouleur(2), false, StyleTrait.traitEpais, point1, point2,
			          StyleFleche.FlecheCourtePleine);
		        vecteurClignotant.positionne(false, appletMere.dimf);
		        ajouteObjetsVisuels();
		        ajouteClignotementDe(elg);
		        ajouteClignotementDe(vecteurClignotant);
		        designationDe(appletMere.natureObjetsPourImages());
		      }
		    }
		  }
	  }
  }
  
}
